package fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;

public class IdentificationDataSourceSet extends
		HashSet<IdentificationDataSource> {

	private static final long serialVersionUID = -4351959247611214080L;

	private static final Logger logger = Logger
			.getLogger(IdentificationDataSourceSet.class);

	private HashMap<String, IdentificationDataSource> stringToDatasource = new HashMap<String, IdentificationDataSource>();

	public IdentificationDataSourceSet() {
	}

	public IdentificationDataSource getInstance(String source)
			throws MSMSException {
		// If exist, return instance
		IdentificationDataSource dataSource = null;
		try {
			if (stringToDatasource.containsKey(source)) {
				dataSource = stringToDatasource.get(source);
			} else {
				// If not exist create correct instance
				if (XtandemPipelineSession.getInstance()
						.isPROTICportDataSource() == false) {
					logger.debug("datasource :" + source);
					if (source.matches(".*dat$")) {
						dataSource = new IdentificationMascotFile(new File(
								source));
						// }else if(source.matches(".*xml$")){
					} else if (source.matches(".*\\.pep\\.xml$")) {
						dataSource = new IdentificationPepXmlFile(new File(
								source));
						// }else if(source.matches(".*xml$")){
					} else {
						dataSource = new IdentificationXtandemFile(new File(
								source));
					}
				} else {
					logger.debug("PP");
					dataSource = new IdentificationProticPortSource(new Long(
							source));
				}
				stringToDatasource.put(source, dataSource);
				logger.debug("New Datasource :" + source);
			}
		} catch (Exception e) {
			String message = "error instanciating dataSource file " + source+ " name ";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (dataSource == null) {
			String message = "dataSource is null for file " + source;
			logger.error(message);
			throw new MSMSException(message);
		}
		return (dataSource);
	}

	public String getKey(IdentificationDataSource sampleFile)
			throws MSMSException {
		for (String key : stringToDatasource.keySet()) {
			if (stringToDatasource.get(key) == sampleFile) {
				return key;
			}
		}
		String message = "IdentificationDataSource key not found for "
				+ sampleFile.getName();
		logger.error(message);
		throw new MSMSException(message);
	}

}
