package fr.inra.pappso.xtandempipeline.class_msms;

import java.util.HashSet;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpMsSample;

/**
 * an MsRun represents an MS sample. An MS sample could have been identified
 * from multiple Identification data sources (several idenfication files)
 * 
 * @author langella
 * 
 */
public class MsRun implements GrpMsSample {
	private static final Logger logger = Logger.getLogger(MsRun.class);
	private String sampleName;

	private HashSet<IdentificationDataSource> files = new HashSet<IdentificationDataSource>(
			0);

	protected MsRun(String sample_name) {
		logger.debug("MsRun begin " + sample_name);

		sampleName = sample_name;
	}

	public IdentificationDataSource addFile(IdentificationDataSource file) {
		if (files.add(file)) {
			return file;
		}
		for (IdentificationDataSource otherFile : this.files) {
			if (file.equals(otherFile)) {
				return otherFile;
			}
		}
		return null;
	}

	public HashSet<IdentificationDataSource> getIdentificationDataSourceList() {
		HashSet<IdentificationDataSource> result = new HashSet<IdentificationDataSource>();
		for (IdentificationDataSource f : files) {
			result.add(f);
		}
		return result;
	}

	public String getSampleName() {
		return sampleName;
	}

	@Override
	public int hashCode() {
		return sampleName.hashCode();
	}

	@Override
	public boolean equals(Object toCompare) {
		if (this.hashCode() == toCompare.hashCode()) {
			MsRun msRunCompare = (MsRun) toCompare;
			if (this.sampleName.equals(msRunCompare.sampleName)) {
				return true;
			} else {
				logger.debug("msRun COLLISION on " + sampleName
						+ " compared to " + msRunCompare.sampleName);
			}
		}
		return false;
	}

	@Override
	public int compareTo(GrpMsSample arg0) {
		return this.sampleName.compareTo(arg0.getSampleName());
	}

	@Override
	public boolean isIdenticalTo(GrpMsSample sample) {
		return this.equals(sample);
	}

}
