/*******************************************************************************
 * Copyright (c) 2013 Olivier Langella <Olivier.Langella@moulon.inra.fr>
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 ******************************************************************************/
package fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import javax.xml.stream.XMLOutputFactory;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFile;
import fr.inra.pappso.xtandempipeline.output.IndentingXMLStreamWriter;

public class XtandemParamXmlFile {
	private static final Logger logger = Logger
			.getLogger(XtandemParamXmlFile.class);

	private File paramFile;

	public XtandemParamXmlFile(File tmpDir, XtandemModelFile modelXmlFile,
			XtandemDatabaseXmlFile databasexml, PeakListFile pklFile,
			XtandemResultFile result_out) throws MSMSException {
		try {
			if (tmpDir == null) {
				paramFile = File.createTempFile("xtandem_param", ".xml");
			} else {
				paramFile = File
						.createTempFile("xtandem_param", ".xml", tmpDir);
			}
		} catch (IOException e) {
			String message = "temporary file creation failed for in newXtandemParamXmlFile (parameter xml file for xtandem)";
			logger.error(message);
			throw new MSMSException(message);
		}

		try {
			XMLOutputFactory factory = XMLOutputFactory.newInstance();
			// OutputStreamWriter noIndentOutput = new OutputStreamWriter(
			// outputStream, "UTF8");
			// XMLStreamWriter writer =
			// factory.createXMLStreamWriter(noIndentOutput);
			OutputStreamWriter output = new OutputStreamWriter(
					new FileOutputStream(paramFile.getAbsolutePath()), "UTF8");

			IndentingXMLStreamWriter writer = new IndentingXMLStreamWriter(
					factory.createXMLStreamWriter(output));

			writer.writeStartDocument("UTF-8", "1.0");

			// out.write("<bioml label=\"" + result_out.getName() + "\">" +
			// "\n");
			// out.write("<note type=\"heading\">Paths</note>" + "\n");

			// out.write("</bioml>" + "\n");

			writer.writeStartElement("bioml");
			writer.writeAttribute("label", result_out.getName());

			writer.writeStartElement("note");
			writer.writeAttribute("type", "heading");
			writer.writeCharacters("Paths");
			writer.writeEndElement();
			// out.write("<note type=\"input\" label=\"list path, default parameters\">"
			// + modelXmlFile.getAbsolutePath() + "</note>" + "\n");

			if (!modelXmlFile.exists()) {
				String message = "the model file '"
						+ modelXmlFile.getAbsolutePath()
						+ "' does not exists !";
				logger.error(message);
				throw new MSMSException(message);
			}
			writer.writeStartElement("note");
			writer.writeAttribute("type", "input");
			writer.writeAttribute("label", "list path, default parameters");
			writer.writeCharacters(modelXmlFile.getAbsolutePath());
			writer.writeEndElement();
			// out.write("<note type=\"input\" label=\"list path, taxonomy information\">"
			// + databasexml.getAbsolutePath() + "</note>" + "\n");
			if (!databasexml.exists()) {
				String message = "the database file '"
						+ databasexml.getAbsolutePath() + "' does not exists !";
				logger.error(message);
				throw new MSMSException(message);
			}
			writer.writeStartElement("note");
			writer.writeAttribute("type", "input");
			writer.writeAttribute("label", "list path, taxonomy information");
			writer.writeCharacters(databasexml.getAbsolutePath());
			writer.writeEndElement();
			// out.write("<note type=\"input\" label=\"spectrum, path\">"
			// + pklFile.getAbsolutePath() + "</note>" + "\n");

			if (!pklFile.exists()) {
				String message = "the peak list file '"
						+ pklFile.getAbsolutePath() + "' does not exists !";
				logger.error(message);
				throw new MSMSException(message);
			}
			writer.writeStartElement("note");
			writer.writeAttribute("type", "input");
			writer.writeAttribute("label", "spectrum, path");
			writer.writeCharacters(pklFile.getAbsolutePath());
			writer.writeEndElement();
			// out.write("<note type=\"heading\">Protein general</note>" +
			// "\n");
			writer.writeStartElement("note");
			writer.writeAttribute("type", "heading");
			writer.writeCharacters("Protein general");
			writer.writeEndElement();
			// out.write("<note type=\"input\" label=\"protein, taxon\">usedefined</note>"
			// + "\n");
			writer.writeStartElement("note");
			writer.writeAttribute("type", "input");
			writer.writeAttribute("label", "protein, taxon");
			writer.writeCharacters("usedefined");
			writer.writeEndElement();
			// out.write("<note type=\"heading\">Output</note>" + "\n");
			writer.writeStartElement("note");
			writer.writeAttribute("type", "heading");
			writer.writeCharacters("Output");
			writer.writeEndElement();
			// out.write("<note type=\"input\" label=\"output, path\">"
			// + result_out.getAbsolutePath() + "</note>" + "\n");
			writer.writeStartElement("note");
			writer.writeAttribute("type", "input");
			writer.writeAttribute("label", "output, path");
			writer.writeCharacters(result_out.getAbsolutePath());
			writer.writeEndElement();

			writer.writeEndElement();
			writer.writeEndDocument();
			writer.close();
			output.close();
		} catch (Exception e) {
			String message = "ERROR writing X!Tandem infile : "
					+ e.getMessage();
			logger.error(message, e);

			throw new MSMSException(message);
		}

	}

	public String getName() {
		return paramFile.getName();
	}

	public boolean exists() {
		return paramFile.exists();
	}

	public String getAbsolutePath() {
		return paramFile.getAbsolutePath();
	}

	public boolean delete() {
		return paramFile.delete();
	}

}
