package fr.inra.pappso.xtandempipeline.filter_print;

import java.text.Collator;
import java.util.Arrays;
import java.util.Comparator;
import java.util.Locale;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;


import fr.inra.pappso.xtandempipeline.grouping.Group;
import fr.inra.pappso.xtandempipeline.grouping.GroupSet;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class swt_table_base {

	protected Table table;

	protected String[] header;

	protected int[] type_column;

	protected boolean color;

	protected DataType_Base type;
	
	protected static final Logger logger = Logger.getLogger(swt_table_base.class);

	public swt_table_base(Table ta, DataType_Base type) {
		this.type = type;
		this.table = ta;
		header = new String[] { " ", "group", "subgroup",
				"Protein Description", "..." };
		type_column = new int[] { 0, 1, 2, 3 };
	}

	public void execute(GroupSet grouping) throws Exception {
		// tout les groupes
		table.removeAll();
		this.set_table_header();
		for (Group group:grouping.getGroupList()) {
			this.set_table_item(group);
		}
		this.pack_table();
	}

	protected void pack_table() {
		for (int i = 0; i < header.length; i++) {
			if (table.getColumn(i).getWidth() == 0)
				table.getColumn(i).pack();
		}
	}

	protected void set_table_header() throws Exception {
		TableColumn column;
		for (int i = 0; i < header.length; i++) {
			if (table.getColumnCount() > i) {
				column = table.getColumn(i);
			} else {
				column = new TableColumn(table, SWT.NONE);
				column.addListener(SWT.Selection,
						SortListenerFactory.getListener(type_column[i]));
				column.setMoveable(true);
			}
			column.setText(header[i]);
		}
	}

	protected boolean set_table_item(Group group) throws Exception {
		return (true);
	}

	protected String get_subgroup_number(Group group, SubGroup sg) {
		String subgroup = group.getNumber() + ".";
		if (sg.getNumber() < 10)
			subgroup = subgroup + "0" + sg.getNumber();
		else
			subgroup = subgroup + sg.getNumber();
		return (subgroup);
	}
}

class SortListenerFactory implements Listener {
	private Comparator<TableItem> currentComparator = null;

	private Collator col = Collator.getInstance(Locale.getDefault());

	public static final int INT_COMPARATOR = 0;
	public static final int STRING_COMPARATOR = 1;
	public static final int DOUBLE_COMPARATOR = 2;
	public static final int FLOAT_COMPARATOR = 3;

	private SortListenerFactory(int _comp) {
		switch (_comp) {
		case INT_COMPARATOR:
			currentComparator = intComparator;
			break;

		case STRING_COMPARATOR:
			currentComparator = strComparator;
			break;

		case DOUBLE_COMPARATOR:
			currentComparator = doubleComparator;
			break;

		case FLOAT_COMPARATOR:
			currentComparator = floatComparator;
			break;

		default:
			currentComparator = null;
		}
	}

	public static Listener getListener(int _comp) {
		return new SortListenerFactory(_comp);
	}

	private int colIndex = 0;
	private int updown = 1;

	// Integer Comparator
	private Comparator<TableItem> intComparator = new Comparator<TableItem>() {
		public int compare(TableItem arg0, TableItem arg1) {
			TableItem t1 = arg0;
			TableItem t2 = arg1;

			int v1 = Integer.parseInt(t1.getText(colIndex));
			int v2 = Integer.parseInt(t2.getText(colIndex));

			if (v1 < v2)
				return 1 * updown;
			if (v1 > v2)
				return -1 * updown;

			return 0;
		}
	};

	// String Comparator
	private Comparator<TableItem> strComparator = new Comparator<TableItem>() {
		public int compare(TableItem arg0, TableItem arg1) {
			TableItem t1 = arg0;
			TableItem t2 = arg1;

			String v1 = (t1.getText(colIndex));
			String v2 = (t2.getText(colIndex));

			return (col.compare(v1, v2)) * -updown;
		}
	};

	// Double Comparator
	private Comparator<TableItem> doubleComparator = new Comparator<TableItem>() {
		public int compare(TableItem arg0, TableItem arg1) {
			TableItem t1 = arg0;
			TableItem t2 = arg1;

			double v1 = Double.parseDouble(t1.getText(colIndex));
			double v2 = Double.parseDouble(t2.getText(colIndex));

			if (v1 < v2)
				return 1 * updown;
			if (v1 > v2)
				return -1 * updown;

			return 0;
		}
	};

	// Float Comparator
	private Comparator<TableItem> floatComparator = new Comparator<TableItem>() {
		public int compare(TableItem arg0, TableItem arg1) {
			TableItem t1 = arg0;
			TableItem t2 = arg1;

			float v1 = Float.parseFloat(t1.getText(colIndex));
			float v2 = Float.parseFloat(t2.getText(colIndex));

			if (v1 < v2)
				return 1 * updown;
			if (v1 > v2)
				return -1 * updown;

			return 0;
		}
	};

	public void handleEvent(Event e) {
		if (currentComparator != null) {
			updown = (updown == 1 ? -1 : 1);

			TableColumn currentColumn = (TableColumn) e.widget;
			Table table = currentColumn.getParent();

			colIndex = searchColumnIndex(currentColumn);

			table.setRedraw(false);

			TableItem[] items = table.getItems();

			Arrays.sort(items, currentComparator);

			table.setItemCount(items.length);

			for (int i = 0; i < items.length; i++) {
				TableItem item = new TableItem(table, SWT.NONE, i);
				item.setText(getData(items[i]));
				item.setBackground(items[i].getBackground());
				item.setData("group", items[i].getData("group"));
				item.setData("subgroup", items[i].getData("subgroup"));
				item.setData("match", items[i].getData("match"));
				item.setData("peptide", items[i].getData("peptide"));
				item.setData("peptide_checked", items[i].getData("peptide_checked"));				
				item.setChecked(items[i].getChecked());
				items[i].dispose();
			}

			table.setRedraw(true);
//			int sort = updown == 1 ? SWT.UP : SWT.DOWN;
//			table.setSortDirection(sort);
		}
	}

	private String[] getData(TableItem t) {
		Table table = t.getParent();

		int colCount = table.getColumnCount();
		String[] s = new String[colCount];

		for (int i = 0; i < colCount; i++)
			s[i] = t.getText(i);

		return s;

	}

	private int searchColumnIndex(TableColumn currentColumn) {
		Table table = currentColumn.getParent();

		int in = 0;

		for (int i = 0; i < table.getColumnCount(); i++)
			if (table.getColumn(i).equals(currentColumn))
				in = i;

		return in;
	}
}
