package fr.inra.pappso.xtandempipeline.grouping;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;

import org.apache.log4j.Logger;

public class GroupSet {

	HashSet<Group> groupList = new HashSet<Group>(0);

	private static final Logger logger = Logger.getLogger(GroupSet.class);

	public GroupSet() {
		// logger.debug("SubGroup begin");
	}

	public void groupingSubGroupSet(SubGroupSet subGroupSet) throws Exception {
		logger.debug("grouping subgroup set");
		for (SubGroup subGroup : subGroupSet) {
			this.addSubGroup(subGroup);
		}

		logger.debug("Removing Non Informative Sub-Group");
		this.removingNonIformativeGroup();

		logger.debug("Numbering group");
		this.numbering();
	}

	public void numbering() {
		// numerotation
		ArrayList<Group> orderedGroupList = new ArrayList<Group>(groupList);

		Collections.sort(orderedGroupList);
		int i = 1;
		for (Group group : orderedGroupList) {
			group.setNumber(i);
			i++;
			group.getSubGroupSet().numbering();
		}

	}

	public int getNumberOfSubgroups() {
		int result = 0;
		for (Group group : this.groupList) {
			result += group.getSubGroupSet().size();
		}

		return result;
	}

	private void removingNonIformativeGroup()  throws Exception {
		HashSet<Group> toRemove = new HashSet<Group>(0);
		HashSet<Group> toAdd = new HashSet<Group>(0);
		for (Group group : this.groupList) {
			boolean remove = false;
			//remove sequentially sub-group to
			while(group.removeNonInformative()){
				remove=true;
			}
			//There is removed Sub-grouped, check it
			if(remove){
				toAdd.addAll(group.reGroup());
				toRemove.add(group);
			}
		}
		this.groupList.removeAll(toRemove);
		this.groupList.addAll(toAdd);
	}

	protected void addSubGroup(SubGroup subGroup) throws Exception {
		logger.debug("addSubGroup "
				+ subGroup.getHashSampleScanSetProtList().get(0).toString());
		Group group = new Group(subGroup);

		this.add(group);
	}

	private boolean add(Group groupToAdd) throws Exception {
		HashSet<Group> removeGroupList = new HashSet<Group>(0);
		boolean fusion = false;
		for (Group group : groupList) {
			if (groupToAdd.fusionWith(group)) {
				fusion = true;
				removeGroupList.add(group);
			}
		}
		if (fusion) {
			logger.debug("fusioning");
			this.groupList.add(groupToAdd);
			this.groupList.removeAll(removeGroupList);
		} else {
			this.groupList.add(groupToAdd);
		}
		return true;

	}

	public ArrayList<Group> getGroupList() {
		ArrayList<Group> orderedGroupList = new ArrayList<Group>(groupList);

		Collections.sort(orderedGroupList);
		return orderedGroupList;
	}

	public HashSet<Group> getHashSetOfGroups() {
		return this.groupList;
	}

	public int size() {
		return this.groupList.size();
	}
}
