/*******************************************************************************
 * Copyright (c) 2012 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Benoit Valot <Benoit.Valot@moulon.inra.fr>
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.ModifsList;

public class MassChroQmlOutput {
	private static final Logger logger = Logger
			.getLogger(MassChroQmlOutput.class);

	ModifsList isotopic_mods;
	Identification ident;

	public MassChroQmlOutput(ModifsList isotopic_mods, Identification ident)
			throws ProticPortException {

		this.isotopic_mods = isotopic_mods;
		this.ident = ident;
	}

	public void write(OutputStream outputStream) {
		try {
			XMLOutputFactory factory = XMLOutputFactory.newInstance();
			// OutputStreamWriter noIndentOutput = new OutputStreamWriter(
			// outputStream, "UTF8");
			// XMLStreamWriter writer =
			// factory.createXMLStreamWriter(noIndentOutput);
			OutputStreamWriter output = new OutputStreamWriter(outputStream,
					"UTF8");

			XMLStreamWriter writer = new IndentingXMLStreamWriter(
					factory.createXMLStreamWriter(output));
			writer.writeStartDocument("UTF-8", "1.0");

			MassChroQmlWriter masschroq_ml_writer = new MassChroQmlWriter(
					writer);
			masschroq_ml_writer.write(this.isotopic_mods, this.ident);
			writer.writeEndDocument();

			writer.flush();
			writer.close();
			outputStream.flush();

			logger.debug("No errors writing MassChroQml. All seems OK!");
		} catch (XMLStreamException e) {
			try {
				outputStream.write(e.getMessage().toString().getBytes());
			} catch (IOException e1) {
				logger.debug("ERROR : " + e.getMessage(), e);
			}
			// throw new ProticPortException ("ERROR writing mzXml "+
			// e.getMessage());
			logger.debug("ERROR writing MassChroQml : " +e.getLocation()+ " " + e.getMessage(), e);

		} catch (Exception e) {
			try {
				outputStream.write(e.getMessage().toString().getBytes());
			} catch (IOException e1) {
				logger.debug("ERROR : " + e.getMessage(), e);
			}
			// throw new ProticPortException ("ERROR writing mzXml "+
			// e.getMessage());
			logger.debug("ERROR writing MassChroQml : " + e.getMessage(), e);
		}
	}
}
