/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output;

import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;
import javax.xml.stream.XMLStreamWriter;

import org.apache.log4j.Logger;

import fr.inra.moulon.proticport.model.exceptions.ProticPortException;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;

public class PepnovoMlOutput {
	private static final Logger logger = Logger
			.getLogger(PepnovoMlOutput.class);

	XMLStreamWriter writer;
	OutputStream outputStream;
	PepnovoMlWriter pepnovo_writer;

	public PepnovoMlOutput() throws ProticPortException {
	}

	public void open(OutputStream outputStream) throws MSMSException {
		XpipFile xpip = XtandemPipelineSession.getInstance()
				.getCurrentXpipFile();
		if (xpip == null) {
			String message = "Please save your work to an XPIP file before proceeding.";
			logger.error(message);
			throw new MSMSException(message);
		}
		try {

			this.outputStream = outputStream;
			XMLOutputFactory factory = XMLOutputFactory.newInstance();
			// OutputStreamWriter noIndentOutput = new OutputStreamWriter(
			// outputStream, "UTF8");
			// XMLStreamWriter writer =
			// factory.createXMLStreamWriter(noIndentOutput);
			OutputStreamWriter output = new OutputStreamWriter(outputStream,
					"UTF8");

			writer = new IndentingXMLStreamWriter(
					factory.createXMLStreamWriter(output));
			writer.writeStartDocument("UTF-8", "1.0");

			pepnovo_writer = new PepnovoMlWriter(writer);

			logger.debug("No errors writing Pepnovo. All seems OK!");
		} catch (Exception e) {
			try {
				outputStream.write(e.getMessage().toString().getBytes());
			} catch (IOException e1) {
				logger.debug("ERROR : " + e.getMessage(), e);
			}
			// throw new ProticPortException ("ERROR writing mzXml "+
			// e.getMessage());
			logger.debug("ERROR writing Pepnovo : " + e.getMessage(), e);
		}
	}

	public void close() {
		try {
			pepnovo_writer.close();
			outputStream.flush();

			logger.debug("No errors writing Pepnovo. All seems OK!");
		} catch (Exception e) {
			try {
				outputStream.write(e.getMessage().toString().getBytes());
			} catch (IOException e1) {
				logger.debug("ERROR : " + e.getMessage(), e);
			}
			// throw new ProticPortException ("ERROR writing mzXml "+
			// e.getMessage());
			logger.debug("ERROR writing Pepnovo : " + e.getMessage(), e);
		}
	}

	public void write(Identification ident) throws XMLStreamException,
			MSMSException {
		pepnovo_writer.write(ident);

	}
}
