/*******************************************************************************
 * Copyright (c) 2014 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTandemPipeline.
 *
 *     XTandemPipeline is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTandemPipeline is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTandemPipeline.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr>
 ******************************************************************************/

package fr.inra.pappso.xtandempipeline.output.SpreadSheets;

import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.HashSet;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.stream.XMLStreamException;

import org.apache.log4j.Logger;

import fr.inra.pappso.libodsstream.OdsDocWriter;
import fr.inra.pappso.libodsstream.OdsDocWriterInterface;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MsRun;
import fr.inra.pappso.xtandempipeline.class_msms.MsRunSet;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;

public class OdsMsSampleInformationOutput {
	private static final Logger logger = Logger
			.getLogger(OdsMsSampleInformationOutput.class);

	protected OdsDocWriterInterface odsTable;

	protected String sheetName = "MS samples informations";

	private boolean xtandem_info_extract = true;

	private boolean mzxml_info_extract = true;

	protected OdsMsSampleInformationOutput() {

	}

	public OdsMsSampleInformationOutput(OutputStream outputStream)
			throws MSMSException {
		try {
			odsTable = new OdsDocWriter(outputStream);
			// X!TandemPipeline informations
			odsTable.writeSheet(sheetName);
			initializeOutputPreferences();
		} catch (IOException e) {
			String message = "IOException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (DatatypeConfigurationException e) {
			String message = "DatatypeConfigurationException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	private void initializeOutputPreferences() {
		config conf = XtandemPipelineSession.getInstance().getConfig();
		if (conf.get_export_raw_ms_sample_stat_spreadsheet()) {
			this.xtandem_info_extract = true;
			this.mzxml_info_extract = true;
		} else {
			this.xtandem_info_extract = false;
			this.mzxml_info_extract = false;
		}

	}

	public OdsMsSampleInformationOutput(OdsDocWriterInterface odsTable)
			throws MSMSException {
		try {
			this.odsTable = odsTable;
			odsTable.writeSheet(sheetName);
			initializeOutputPreferences();
		} catch (XMLStreamException e) {
			String message = "XMLStreamException " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception " + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void close() throws MSMSException {
		try {
			odsTable.close();
		} catch (Exception e) {
			String message = "Exception closing " + sheetName + "\n"
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void writeXtpSessionInformations(DataType_Base datatype,
			ArrayList<Identification> identificationList) throws MSMSException {
		try {

			odsTable.writeLine();
			odsTable.writeCell("MS sample name");
			odsTable.writeCell("Number of groups");
			odsTable.writeCell("Number of subgroups");
			if (xtandem_info_extract) {
				odsTable.writeCell("X!tandem xml result file name");
				odsTable.writeCell("X!Tandem version");
				odsTable.writeCell("X!tandem xml model file name");
				odsTable.writeCell("total spectra used");
				odsTable.writeCell("total spectra assigned");
				odsTable.writeCell("assigned/used percent");
				odsTable.writeCell("total unique assigned");
				odsTable.writeCell("database file names");
				odsTable.writeCell("MS/MS data source file name");
				odsTable.writeCell("MS/MS data source file path");
				if (mzxml_info_extract) {
					odsTable.writeCell("MS level 1");
					odsTable.writeCell("MS level 2");
					odsTable.writeCell("TIC mean in MS 1");
					odsTable.writeCell("TIC mean in MS 2");
					odsTable.writeCell("rt min");
					odsTable.writeCell("rt max");
				}
			}

			// int index = 0;
			// XtandemPipelineSession xtp_session =
			// XtandemPipelineSession.getInstance();
			// Boolean individual_mode =
			// xtp_session.getDataTypeBase().getIs_invididual();

			for (Identification identification : identificationList) {
				MsRunSet msrunList = identification.getMsRunSet();
				for (MsRun msrun : msrunList) {
					HashSet<IdentificationDataSource> dataSourceList = msrun
							.getIdentificationDataSourceList();
					for (IdentificationDataSource dataSource : dataSourceList) {
						odsTable.writeLine();
						odsTable.writeCell(msrun.getSampleName());

						// number of groups :
						odsTable.writeCell(identification.getGrouping()
								.getGroupList().size());
						// number of subgroups :
						odsTable.writeCell(identification.getGrouping()
								.getNumberOfSubgroups());

						dataSource.writeOdsMsSampleInformations(odsTable,
								xtandem_info_extract, mzxml_info_extract);
					}
				}
			}

		} catch (XMLStreamException e) {
			String message = "XMLStreamException in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		} catch (Exception e) {
			String message = "Exception in write(Identification ident) "
					+ e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}
}
