package fr.inra.pappso.xtandempipeline.sax_parse;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.log4j.Logger;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.Modifs;
import fr.inra.pappso.xtandempipeline.class_msms.Protein;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.PeptideProphet;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.PeptideStore;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;
import fr.inra.pappso.xtandempipeline.thread.XtpProcess;

public class HandlerXpipFile extends SaxBaseHandler {
	private static final Logger logger = Logger
			.getLogger(HandlerXpipFile.class);

	private Protein prot_;

	private Peptide currentPeptide = null;

	private Identification identification_;

	// private ArrayList<Identification> identifications;

	private Match match_;

	private DataType_Base datatype;

	private XtpProcess pro;

	private String version;

	public HandlerXpipFile(config conf, XtpProcess p) {
		super();
		// identifications =
		// XtandemPipelineSession.getInstance().getIdentificationList();
		datatype = XtandemPipelineSession.getInstance().getDataTypeBase();
		this.pro = p;
	}

	@Override
	public void startDocument() throws SAXException {
		identification_ = new Identification();
		prot_ = null;
		// pep_ = PeptideStore.getInstance().newPeptideId();
		match_ = new Match();
		version = null;
		logger.info("Load project ...");
	}

	@Override
	public void endDocument() throws SAXException {
		logger.info("Load project finish");
	}

	public void startElementxtandem_pipeline(String nameSpaceUri,
			Attributes attrs) {
		logger.debug("xtandem_pipeline");
		this.version = attrs.getValue("version");
	}

	public void startElementinformation(String nameSpaceUri, Attributes attrs) {
		logger.debug("information");
		datatype = datatype.getType(attrs.getValue("Data_Type"));
		XtandemPipelineSession.getInstance().setDataTypeBase(datatype);
		this.pro.set_max_progress(Integer.parseInt(attrs
				.getValue("match_number")));
	}

	public void startElementfilter_params(String nameSpaceUri, Attributes attrs) {
		logger.debug("filter_params");
		datatype.getConf().set_peptide_evalue(
				Float.valueOf(attrs.getValue("pep_evalue")));
		datatype.getConf().set_protein_evalue(
				Float.valueOf(attrs.getValue("prot_evalue")));
		datatype.getConf().set_peptide_number(
				Integer.parseInt(attrs.getValue("pep_number")));
		datatype.getConf().set_protein_filter_to_complete(
				attrs.getValue("filter_to_all").equals("true"));
		datatype.getConf().set_database_filter(
				new File(attrs.getValue("database_filter")));
	}

	public void startElementidentification(String nameSpaceUri, Attributes attrs) {
		logger.debug("identification");
		identification_ = new Identification();
	}

	public void startElementsample(String nameSpaceUri, Attributes attrs) {
		identification_.getMsRunSet().getInstanceBySampleName(
				attrs.getValue("value"));
	}

	public void startElementmatch(String nameSpaceUri, Attributes attrs) {
		match_ = new Match();
		match_.setIsvalidate(attrs.getValue("validate").equals("true"));
	}

	public void startElementprotein_evalue(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		String sampleName = attrs.getValue("sample");
		if (sampleName == null) {
			throw new MSMSException("sample attribute is null");
		}
		prot_.set_evalue(
				identification_.getMsRunSet().getInstanceBySampleName(
						sampleName), Float.valueOf(attrs.getValue("evalue"))
						.floatValue());
	}

	public void startElementmodifs_mass(String nameSpaceUri, Attributes attrs) {
		float modvalue = Float.valueOf(attrs.getValue("modvalue"));
		identification_.add_modifs_to_liste(new Modifs(modvalue));
	}

	public void startElementPROTICport(String nameSpaceUri, Attributes attrs)
			throws MSMSException, MalformedURLException {
		logger.debug("startElementPROTICport begin");

		if (attrs.getValue("url") == null) {
			throw new MSMSException("url is null");
		}
		URL url = new URL(attrs.getValue("url"));
		String login = attrs.getValue("user");
		String password = attrs.getValue("password");
		XtandemPipelineSession.getInstance().setPROTICportDataSource();
		XtandemPipelineSession.getInstance().setPROTICportController(url,
				login, password);

	}

	public void startElementmodif(String namespaceURI, Attributes attrs)
			throws MSMSException {
		try {
			String aa = attrs.getValue("aa");
			if (aa == null) {
				throw new MSMSException("aa is null");
			}
			if (attrs.getValue("modvalue") == null) {
				throw new MSMSException("modvalue is null");
			}
			float modvalue = Float.valueOf(attrs.getValue("modvalue"))
					.floatValue();
			if (attrs.getValue("posi_in_prot") == null) {
				throw new MSMSException("posi_in_prot is null");
			}
			int posi_in_prot = Integer.parseInt(attrs.getValue("posi_in_prot"));
			Peptide peptide = currentPeptide;
			peptide.set_Modifs(aa, posi_in_prot, modvalue);
		} catch (Exception e) {
			String message = "ERROR reading modif tag :\n" + e.getMessage();
			logger.error(message);
			throw new MSMSException(message);
		}
	}

	public void startElementpeptide(String namespaceURI, Attributes attrs)
			throws MSMSException, SAXException {
		if (pro.isStop()) {
			throw new SAXException("sax parser stopped");
		}

		logger.debug("create peptideId");
		currentPeptide = PeptideStore.getInstance().newPeptide();
		logger.debug("get peptide");
		Peptide peptide = currentPeptide;
		if (peptide == null) {
			throw new MSMSException("peptide is null");
		}
		// ajouter les elements du peptides
		String originalFileName = attrs.getValue("sample_file");
		if (originalFileName == null) {
			throw new MSMSException("sample_file is null");
		}
		String sampleName = attrs.getValue("sample");
		if (sampleName == null) {
			throw new MSMSException("sample is null");
		}
		logger.debug("IdentificationDataSource sample_file");
		IdentificationDataSource sample_file = XtandemPipelineSession
				.getInstance().getIdentificationDataSourceInstance(
						originalFileName);
		peptide.set_sample_id(this.identification_.getMsRunSet()
				.getInstanceBySampleName(sampleName), sample_file);

		// read scan number :
		if (attrs.getValue("scan") == null) {
			throw new MSMSException("scan is null");
		}
		peptide.set_scan(Integer.parseInt(attrs.getValue("scan")));
		if (attrs.getValue("scan_in_xtandem") != null) {
			// if there is a correspondance to make in the source file :
			sample_file.setScanCorrespondance(peptide.get_scan(),
					Integer.parseInt(attrs.getValue("scan_in_xtandem")));
		}

		if (attrs.getValue("RT") == null) {
			// throw new MSMSException("RT is null");
			logger.warn("RT is null");
		} else {
			peptide.set_RT(attrs.getValue("RT"));
		}
		if (attrs.getValue("mhplus_obser") == null) {
			throw new MSMSException("mhplus_obser is null");
		}
		peptide.set_mhplus_obser(Float.valueOf(attrs.getValue("mhplus_obser"))
				.floatValue());
		if (attrs.getValue("mhplus_theo") == null) {
			throw new MSMSException("mhplus_theo is null");
		}
		peptide.set_mhplus_theo(Float.valueOf(attrs.getValue("mhplus_theo"))
				.floatValue());
		if (attrs.getValue("deltamass") == null) {
			throw new MSMSException("deltamass is null");
		}
		peptide.set_deltamass(Float.valueOf(attrs.getValue("deltamass"))
				.floatValue());
		if (attrs.getValue("sequence") == null) {
			throw new MSMSException("sequence is null");
		}
		peptide.set_sequence(attrs.getValue("sequence"));
		if (attrs.getValue("pre") == null) {
			throw new MSMSException("pre is null");
		}
		peptide.set_pre(attrs.getValue("pre"));
		if (attrs.getValue("post") == null) {
			throw new MSMSException("post is null");
		}
		peptide.set_post(attrs.getValue("post"));
		if (attrs.getValue("start") == null) {
			throw new MSMSException("start is null");
		}
		peptide.set_start(Integer.parseInt(attrs.getValue("start")));
		if (attrs.getValue("stop") == null) {
			throw new MSMSException("stop is null");
		}
		peptide.set_stop(Integer.parseInt(attrs.getValue("stop")));
		if (attrs.getValue("charge") == null) {
			throw new MSMSException("charge is null");
		}
		peptide.set_charge(Integer.parseInt(attrs.getValue("charge")));
		if (attrs.getValue("evalue") == null) {
			throw new MSMSException("evalue is null");
		}
		peptide.set_evalue(Float.valueOf(attrs.getValue("evalue")).floatValue());
		if (attrs.getValue("hypercorr") == null) {
			throw new MSMSException("hypercorr is null");
		}
		peptide.set_hypercorr(Float.valueOf(attrs.getValue("hypercorr"))
				.floatValue());
		if (attrs.getValue("validate") == null) {
			throw new MSMSException("validate is null");
		}
		peptide.setIsValid(attrs.getValue("validate").equals("true"));
		// ajouter le peptide au match
		logger.debug("add peptideId");
		match_.add_peptide_match(this.currentPeptide);
		identification_.add_peptide(this.currentPeptide);

	}

	public void startElementprophet(String nameSpaceUri, Attributes attrs)
			throws MSMSException {
		PeptideProphet pepProphet = new PeptideProphet(currentPeptide);
		if (attrs.getValue("prophet_prob") != null) {
			pepProphet.setProphetProbability(Double.parseDouble(attrs
					.getValue("prophet_prob")));
		}
		if (attrs.getValue("inter_prophet_prob") != null) {
			pepProphet.setInterProphetProbability(Double.parseDouble(attrs
					.getValue("inter_prophet_prob")));
		}
		pepProphet.setSearchEngine(attrs.getValue("search_engine"));
		currentPeptide = pepProphet;

	}

	public void startElementprotein(String nameSpaceUri, Attributes attrs)
			throws MSMSException {

		if (prot_ != null) {
			throw new MSMSException(
					"more than one protein in this match element");
		}
		prot_ = new Protein();
		// ajouter les elements de la prot
		if (attrs.getValue("description") == null) {
			throw new MSMSException("description is null");
		}
		prot_.set_description(attrs.getValue("description"));
		if (attrs.getValue("evalue") == null) {
			throw new MSMSException("evalue is null");
		}
		prot_.set_evalue(Float.valueOf(attrs.getValue("evalue")).floatValue());
		if (attrs.getValue("URL") == null) {
			throw new MSMSException("URL is null");
		}
		prot_.setDatabase(identification_.getDatabaseSet().getInstance(
				attrs.getValue("URL")));
		if (attrs.getValue("peptide_number") == null) {
			throw new MSMSException("peptide_number is null");
		}

	}

	public void endElementprotein(String nameSpaceUri, String data) {
		match_.set_protein_match(prot_);
	}

	public void endElementsequence(String nameSpaceUri, String data) {
		prot_.set_sequence(data);
	}

	public void endElementmatch(String nameSpaceUri, String data) {
		identification_.add_protein(prot_, match_);
		this.pro.add_current_progress();
		prot_ = null;
	}

	public void endElementidentification(String nameSpaceUri, String data) {
		XtandemPipelineSession.getInstance().getIdentificationList()
				.add(identification_);
	}

	public String getVersion() {
		return version;
	}
}
