package fr.inra.pappso.xtandempipeline.sax_parse;

import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.class_msms.xtandem_param;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;
import fr.inra.pappso.xtandempipeline.output.XtandemParamsOutput;

public class xtandem_params {

	private static final Logger logger = Logger.getLogger(xtandem_params.class);

	private Vector<xtandem_param> params;

	private Vector<String> groups;

	public xtandem_params() throws Exception {
		this.params = new Vector<xtandem_param>();
		this.groups = new Vector<String>();
		this.load_new_param();
	}

	public void load_xml(XtandemModelFile xml) throws Exception {
		logger.info("Load xtandem param from : " + xml.getName());

		xml.readParamsFromFile(this);

	}

	private void load_new_param() throws Exception {

		InputStream template = this.getClass().getResourceAsStream(
				"/resources/xtandem_model.xml");
		if (template == null) {
			throw new MSMSException(
					"Problem accessing resource : xtandem_model.xml");
		}
		logger.debug("load_new_param begin");
		HandlerXtModelFile4newParams handler = new HandlerXtModelFile4newParams(
				this);
		logger.debug("load_new_param parse_file");
		new parse_file(handler, template);
		logger.debug(this.groups.size());
	}

	public void add_params(xtandem_param p) {
		this.params.add(p);
		if (!this.groups.contains(p.getGroup()))
			this.groups.add(p.getGroup());
	}

	public Vector<xtandem_param> get_params_to_group(String g) {
		Vector<xtandem_param> x = new Vector<xtandem_param>();
		for (xtandem_param param : this.params) {
			if (param.getGroup().equals(g)) {
				x.add(param);
			}
		}
		return x;
	}

	public xtandem_param get_param_to(String note_value) {
		for (xtandem_param param : this.params) {
			if (param.xmlvalue().equalsIgnoreCase(note_value)) {
				return (param);
			}
		}
		return (null);
	}

	public Vector<String> get_groups() {
		return (groups);
	}

	public void save_to_xml(XtandemModelFile xml) throws Exception {

		new XtandemParamsOutput(new FileOutputStream(xml.getAbsolutePath()),
				this.params);
	}

}
