package fr.inra.pappso.xtandempipeline.thread;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;

public class filtered_and_create_grouping extends XtpProcess {
	protected static final Logger logger = Logger
			.getLogger(filtered_and_create_grouping.class);

	private Identification ident;

	private DataType_Base data_type;

	public filtered_and_create_grouping(config conf, Identification ident,
			DataType_Base data_type) {
		super(conf);
		this.ident = ident;
		this.data_type = data_type;
		this.set_max_progress(4);
	}

	protected void processing() throws Exception {
		logger.info("Filtered result and create grouped");

		// restreindre la liste des protéines aux protéines identifiées
		this.set_view("Removing redondant proteins");
		ident.setProteinListValideToGroupeSet();
		this.add_current_progress();

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");

		this.set_view("Filtering proteins");
		// recalcul de la evalue proteique
		ident.filtered_match(new filter_recalcul_protein_evalue());

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		if (conf.get_protein_filter_to_complete()) {
			ident.filtered_match(new filter_protein_evalue(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number(conf
					.get_peptide_number()));
		} else {
			ident.filtered_match(new filter_protein_evalue_to(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number_to(conf
					.get_peptide_number()));
		}
		this.add_current_progress();

		// creation des submatchs
		this.set_view("Grouping \n"
				+ "Be patient, this step can take some time");
		if (data_type.is_phosphopeptide) {
			ident.performedPhosphoGroupSet();
		} else {
			ident.performedGroupSet();
		}

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	public void setData_type(DataType_Base data_type) {
		this.data_type = data_type;
	}

	public DataType_Base getData_type() {
		return data_type;
	}

	public Identification getIdent() {
		return ident;
	}

	public void setIdent(Identification ident) {
		this.ident = ident;
	}

}
