package fr.inra.pappso.xtandempipeline.thread;

import java.util.ArrayList;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.filter_print.filter_index_and_order_match_scan;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;
import fr.inra.pappso.xtandempipeline.sax_parse.HandlerXpipFile;

public class load_result_and_create_grouping extends XtpProcess {
	private static final Logger logger = Logger
			.getLogger(load_result_and_create_grouping.class);

	// private ArrayList<Identification> identifications;

	// private DataType_Base data_type;

	private XpipFile infile;

	private String version;

	/*
	 * public load_result_and_create_grouping(config conf, XpipFile xpipFile)
	 * throws MSMSException { super(conf); this.set_max_progress(0);
	 *
	 * infile = xpipFile;
	 * logger.debug("load_result_and_create_grouping constructor"); }
	 */
	public load_result_and_create_grouping(config conf, XpipFile xpipFile)
			throws MSMSException {
		super(conf);
		this.set_max_progress(0);

		try {
			infile = xpipFile;
		} catch (Exception e) {
			throw new MSMSException("XtandemPipeline project file "
					+ xpipFile.getAbsolutePath() + " not found");
		}
		logger.debug("load_result_and_create_grouping constructor");
	}

	protected void processing() throws Exception {
		logger.debug("Load project and create grouping");
		// restreindre la liste des protéines aux protéines identifiées
		// this.set_view("Loading data from file :\n" + infile);
		this.set_view("Loading data from file :\n");
		HandlerXpipFile load = new HandlerXpipFile(conf, this);
		// logger.debug("infile available : " + infile.available());

		this.infile.read(load);

		// On teste si nouvelle version et si compatible, sinon on lance une
		// erreur
		DataType_Base data_type = XtandemPipelineSession.getInstance()
				.getDataTypeBase();
		if (!data_type.getIs_active()) {
			throw new MSMSException(
					"This old version of xtandempipeline project is not compatible");
		}
		this.version = load.getVersion();
		if (data_type.getIs_phosphopeptide() && this.version == null) {
			throw new MSMSException(
					"Phosphopeptide analysis have been modified and are not compatible with old version of  xtandempipeline project");
		}

		ArrayList<Identification> identifications = XtandemPipelineSession
				.getInstance().getIdentificationList();
		if (identifications.size() == 0) {
			throw new MSMSException(
					"This xtandempipeline project file contains no identification results");
		}

		XtandemPipelineSession.getInstance().setDataTypeBase(data_type);

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.set_current_progress(0);
		this.set_max_progress(3 * identifications.size());

		for (Identification ident : identifications) {
			// on mets à jour les liens
			ident.filtered_match(new filter_index_and_order_match_scan());

			this.set_view("Filtering proteins");
			// recalcul de la evalue proteique
			ident.filtered_match(new filter_recalcul_protein_evalue());

			// verification pas arreté
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();

			if (conf.get_protein_filter_to_complete()) {
				ident.filtered_match(new filter_protein_evalue(conf
						.get_protein_evalue()));
				ident.filtered_match(new filter_unique_peptide_number(conf
						.get_peptide_number()));
			} else {
				ident.filtered_match(new filter_protein_evalue_to(conf
						.get_protein_evalue()));
				ident.filtered_match(new filter_unique_peptide_number_to(conf
						.get_peptide_number()));
			}
			this.add_current_progress();

			// creation des submatchs
			this.set_view("Grouping \n"
					+ "Be patient, this step can take some time");
			if (data_type.is_phosphopeptide) {
				ident.performedPhosphoGroupSet();
			} else {
				ident.performedGroupSet();
			}

			// verification pas arreté
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();

		}
		this.add_current_progress();
	}

	public String getVersion() {
		return version;
	}
}
