package fr.inra.pappso.xtandempipeline.thread;

import java.util.ArrayList;
import java.util.Vector;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.IdentificationDataFile.IdentificationDataSource;
import fr.inra.pappso.xtandempipeline.filter_print.filter_index_and_order_match_scan;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_redondancy;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_protein_evalue_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_recalcul_protein_evalue;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number;
import fr.inra.pappso.xtandempipeline.filter_print.filter_unique_peptide_number_to;
import fr.inra.pappso.xtandempipeline.filter_print.filter_peptide_evalue;

public class phosphosite extends XtpProcess {
	protected static final Logger logger = Logger.getLogger(phosphosite.class);

	private Vector<IdentificationDataSource> dataSources;

	private ArrayList<Identification> identifications = XtandemPipelineSession
			.getInstance().getIdentificationList();

	public phosphosite(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Process result in Phopho mode");
		// Ecrire le process et penser à relever des StopException
		// XtandemPhospho handler = new XtandemPhospho();
		Identification ident = null;
		for (int i = 0; i < dataSources.size(); i++) {
			// parsing
			this.set_view("Parsing file : " + (i + 1) + "/"
					+ dataSources.size() + "\n" + dataSources.get(i).getName());
			// handler.set_sample(dataSources.get(i));
			// new parse_file(handler, dataSources.get(i));
			ident = dataSources.get(i).loadPhosphoIdentification(ident);

			// verification pas arreté
			if (this.isStop())
				throw new StopException("");
			this.add_current_progress();
		}

		// filtering
		this.set_view("Parsing finished\n" + "Filtering peptides");
		// Identification ident = handler.get_identifications();

		// on mets à jour les liens
		ident.filtered_match(new filter_index_and_order_match_scan());

		// filtre evalue peptidique
		ident.filtered_match(new filter_peptide_evalue(conf
				.get_peptide_evalue(), conf
				.getMinimumPeptideProphetProbability(), conf
				.getMinimumPeptideInterProphetProbability()));
		ident.filtered_match(new filter_peptide_redondancy());

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// calcul de la evalue globale et modification de la evalue sample
		ident.filtered_match(new filter_recalcul_protein_evalue());

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		this.set_view("Parsing finished\n" + "Filtering proteins");
		if (conf.get_protein_filter_to_complete()) {
			ident.filtered_match(new filter_protein_evalue(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number(conf
					.get_peptide_number()));
		} else {
			ident.filtered_match(new filter_protein_evalue_to(conf
					.get_protein_evalue()));
			ident.filtered_match(new filter_unique_peptide_number_to(conf
					.get_peptide_number()));
		}

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// creation des submatchs
		this.set_view("Grouping\n" + "Be patient, this step can take some time");
		ident.performedPhosphoGroupSet();

		// verification pas arreté
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

		// elimination des protéines contaminantes
		ident.remove_contaminant_submatch(conf.get_database_filter());

		identifications.add(ident);

		// Verification pas arrete
		if (this.isStop())
			throw new StopException("");
		this.add_current_progress();

	}

	@Override
	public void setDataSource(Vector<IdentificationDataSource> dataSource) {
		this.dataSources = dataSource;
		this.set_max_progress(dataSources.size() + 7);
	}

}
