package fr.inra.pappso.xtandempipeline.thread;

import java.io.File;

import org.apache.log4j.Logger;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.FastaFileSet;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFile;
import fr.inra.pappso.xtandempipeline.class_msms.PeakListFileSet;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemDatabaseXmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemModelFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemParamXmlFile;
import fr.inra.pappso.xtandempipeline.class_msms.XtandemFiles.XtandemResultFile;
import fr.inra.pappso.xtandempipeline.sax_parse.xtandem_params;

public class xtandem_analysis extends XtpProcess {
	protected static final Logger logger = Logger
			.getLogger(xtandem_analysis.class);

	protected PeakListFileSet peakListFiles;

	protected FastaFileSet databases;

	private File save_dir;

	protected XtandemModelFile model;

	protected File tmpDir = null;

	public xtandem_analysis(config conf) {
		super(conf);
	}

	protected void processing() throws Exception {
		logger.info("Xtandem analysis start");
		// creation du fichier database
		XtandemDatabaseXmlFile databasexml = new XtandemDatabaseXmlFile(
				this.tmpDir, this.databases);

		// Correction du fichier de parametres
		logger.info("Corrected xtandem preset error");
		xtandem_params params = new xtandem_params();
		params.load_xml(model);
		params.save_to_xml(model);

		int i = 0;
		for (PeakListFile pklFile : peakListFiles) {
			i++;
			logger.info("Xtandem analysis on : " + pklFile.getName());
			XtandemResultFile result_out = new XtandemResultFile(
					this.getSave_dir(), pklFile);
			XtandemParamXmlFile infile = new XtandemParamXmlFile(this.tmpDir,
					model, databasexml, pklFile, result_out);

			// creation des fichiers
			base_shell pipe = new base_shell();
			pipe.set_commande(new String[] {
					conf.getXtandem_link().getAbsolutePath(),
					infile.getAbsolutePath() });
			pipe.start_process();
			while (pipe.status()) {
				Thread.currentThread();
				Thread.sleep(100);
				if (this.isStop()) {
					pipe.stop_process();
					throw new StopException("");
				}
				this.set_view("Analyze file : " + i + "/"
						+ peakListFiles.size() + "\n" + pklFile.getName()
						+ "\n" + pipe.get_Stdout());
				// System.out.println(pipe.getStatus());

			}
			if (pipe.get_error() != null)
				throw pipe.get_error();
			if (!result_out.exists())
				throw new MSMSException("Analysis of '" + pklFile.getName()
						+ " gave no result'\n" + pipe.get_Stdout());

			logger.info("finish");
			infile.delete();
			this.add_current_progress();
		}
		databasexml.delete();

	}

	public void setPeakListFileSet(PeakListFileSet peakListFiles) {
		this.peakListFiles = peakListFiles;
		this.set_max_progress(peakListFiles.size());
	}

	public PeakListFileSet getPeakListFileSet() {
		return peakListFiles;
	}

	public void setDatabases(FastaFileSet files) {
		this.databases = files;
	}

	public FastaFileSet getDatabases() {
		return databases;
	}

	public void setSave_dir(File save_dir) throws MSMSException {
		if (save_dir == null) {
			String message = "save_dir is null";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (!save_dir.exists()) {
			String message = "save_dir " + save_dir.getAbsolutePath()
					+ " does not exists";
			logger.error(message);
			throw new MSMSException(message);
		}
		if (!save_dir.isDirectory()) {
			String message = "save_dir " + save_dir.getAbsolutePath()
					+ " is not a directory";
			logger.error(message);
			throw new MSMSException(message);
		}
		this.save_dir = save_dir;
	}

	public File getSave_dir() {
		return save_dir;
	}

	public void setModel(XtandemModelFile model) {
		this.model = model;
	}

	public XtandemModelFile getModel() {
		return model;
	}

}
