package fr.inra.pappso.xtandempipeline.ui.swt;

import fr.inra.pappso.xtandempipeline.Utils;
import fr.inra.pappso.xtandempipeline.XtandemPipelineMain;
import fr.inra.pappso.xtandempipeline.XtandemPipelineSession;
import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.Database;
import fr.inra.pappso.xtandempipeline.class_msms.XpipFile;
import fr.inra.pappso.xtandempipeline.class_msms.config;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_peptide_fdr;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_peptide_mh_deviation;
import fr.inra.pappso.xtandempipeline.filter_print.calculation_protein_fdr;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;
import fr.inra.pappso.xtandempipeline.thread.DataType_Combine;
import fr.inra.pappso.xtandempipeline.thread.DataType_individual;
import fr.inra.pappso.xtandempipeline.thread.DataType_phosphopeptide;
import fr.inra.pappso.xtandempipeline.thread.XtandemCondorAnalysis;
import fr.inra.pappso.xtandempipeline.thread.load_result_and_create_grouping;
import fr.inra.pappso.xtandempipeline.thread.XtpProcess;
import fr.inra.pappso.xtandempipeline.thread.reinitialise_result_filter_and_create_grouping;
import fr.inra.pappso.xtandempipeline.thread.save_result_and_filtering;
import fr.inra.pappso.xtandempipeline.thread.xtandem_analysis;
import fr.inra.pappso.xtandempipeline.ui.results.Peptide_Details;
import fr.inra.pappso.xtandempipeline.ui.results.Peptide_List;
import fr.inra.pappso.xtandempipeline.ui.results.Protein_Details;
import fr.inra.pappso.xtandempipeline.ui.results.Protein_List;
import fr.inra.pappso.xtandempipeline.ui.results.window_base;
import fr.inra.pappso.xtandempipeline.ui.swt.Export_Windows;
import fr.inra.pappso.xtandempipeline.ui.swt.Filter_Windows;
import fr.inra.pappso.xtandempipeline.ui.swt.Running;

import java.awt.Desktop;
import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.ExpandBar;
import org.eclipse.swt.widgets.ExpandItem;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

public class Main_Windows {
	private static final Logger logger = Logger.getLogger(Main_Windows.class);

	private Display display;

	protected Shell shell;

	private Peptide_Details peptide_details;

	private Protein_Details protein_details;

	private Peptide_List peptide_list;

	private Protein_List protein_list;

	protected config conf;

	private Combo identif_select;

	private List database_list;

	private Image[] images;

	private Vector<Button> buttons;

	private Vector<MenuItem> items;

	private Button filter_button;

	protected MenuItem menuItemSaveProjectAs;

	protected MenuItem menuItemSaveProject;

	public Main_Windows(XpipFile in) {
		logger.debug("Main_Windows begin");

		// ouverture d'une session
		boolean oldconfig = false;
		try {
			conf = XtandemPipelineSession.getInstance().getConfig();
			oldconfig = conf.loadPropertiesFromXml();
		} catch (Exception e) {
			logger.error("Problem to load old config file" + e);
			// e.printStackTrace();
		}
		if (in != null) {
			XtandemPipelineSession.getInstance().setCurrentXpipFile(in);
		}
		// System.out.println(conf.getXtandemVersion());
		buttons = new Vector<Button>();
		items = new Vector<MenuItem>();
		peptide_details = new Peptide_Details(this);
		protein_details = new Protein_Details(this);
		peptide_list = new Peptide_List(this);
		protein_list = new Protein_List(this);

		display = new Display();

		InputStream icon16Ins = getClass().getResourceAsStream(
				"/resources/images/icon_16.png");
		InputStream icon32Ins = getClass().getResourceAsStream(
				"/resources/images/icon_32.png");
		InputStream icon64Ins = getClass().getResourceAsStream(
				"/resources/images/icon_64.png");

		final Image small = new Image(display, icon16Ins);
		final Image large = new Image(display, icon32Ins);
		final Image normal = new Image(display, icon64Ins);
		images = new Image[] { small, normal, large };

		shell = new Shell(display);
		shell.setImages(images);

		this.set_window();

		shell.setText("X!TandemPipeline " + XtandemPipelineMain.version + " \""
				+ XtandemPipelineMain.versionName + "\"");

		if (!conf.getCurrent_version().equals(XtandemPipelineMain.version)) {
			MessageBox messageBox = new MessageBox(shell, SWT.OK);
			messageBox.setText("News on version : \""
					+ XtandemPipelineMain.versionName + "\"");
			messageBox.setMessage(XtandemPipelineMain.news);
			messageBox.open();
			conf.setCurrent_version();
		}
		if (!oldconfig)
			new Path_configuration(this);

		if (in != null) {
			load_result_and_create_grouping loading = null;
			try {
				loading = new load_result_and_create_grouping(conf, in);
			} catch (MSMSException e) {
				this.view_error(e, "Problems loading project :");
			}
			this.loadCurrentProject(loading);
		}

		final Main_Windows self = this;
		shell.addListener(SWT.Close, new Listener() {
			public void handleEvent(Event event) {
				if (self.wantToCloseXtandemPipeline() == false) {
					event.doit = false;
				}
			}
		});

		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
		}

	}

	private void set_window() {
		shell.setLayout(new GridLayout(1, false));

		this.set_menu();

		this.set_result_view();

		ExpandBar bar = new ExpandBar(shell, SWT.V_SCROLL);

		bar.setLayoutData(new GridData(GridData.FILL_BOTH));
		this.add_expand_bar_fdr(bar);
		this.add_expand_bar_deltamh(bar);
		this.add_expand_bar_filter(bar);
		bar.setSpacing(8);

		for (Button but : buttons)
			but.setEnabled(false);

		display.addFilter(SWT.Selection, new ResultSelectionListener(this));
		shell.pack();
		shell.setSize(350, 550);
		shell.open();

	}

	private void add_expand_bar_fdr(ExpandBar bar) {
		// TODO Auto-generated method stub
		Composite composite = new Composite(bar, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		composite.setLayout(layout);
		Label l1 = new Label(composite, SWT.NONE);
		l1.setText("Select decoy database");
		l1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
		this.database_list = new List(composite, SWT.MULTI | SWT.V_SCROLL
				| SWT.BORDER);
		database_list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true,
				true, 2, 1));
		database_list.add("");
		database_list.add("");
		database_list.setEnabled(false);

		Button xtandemReverse = new Button(composite, SWT.CHECK);
		xtandemReverse
				.setText("Reverse/Decoy database option \n during X!Tandem or Mascot analysis");
		xtandemReverse.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 2, 1));
		xtandemReverse.setSelection(true);
		xtandemReverse.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Button b = (Button) e.widget;
				database_list.setEnabled(!b.getSelection());
			}
		});

		Text t1 = new Text(composite, SWT.SINGLE);

		GridData col = new GridData(SWT.FILL, SWT.CENTER, false, false);
		col.horizontalAlignment = GridData.FILL;
		col.grabExcessHorizontalSpace = true;
		t1.setText("NA");
		t1.setLayoutData(col);
		Label l2 = new Label(composite, SWT.NONE);
		l2.setText("FDR at peptide level (%):");
		l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t2 = new Text(composite, SWT.SINGLE);
		t2.setText("NA");
		t2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l3 = new Label(composite, SWT.NONE);
		l3.setText("FDR at protein level (%):");
		l3.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button start = new Button(composite, SWT.PUSH);
		start.setText("Estimate");
		start.setData("peptide", t1);
		start.setData("protein", t2);
		start.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					Main_Windows.this.calcul_fdr((Button) e.widget);
				} catch (MSMSException e1) {
					// TODO Auto-generated catch block
					logger.error(e1.getMessage());
					e1.printStackTrace();
				}
			}
		});
		buttons.add(start);

		ExpandItem item0 = new ExpandItem(bar, SWT.NONE, 0);
		item0.setText("False Discovery Rate");
		item0.setHeight(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		item0.setControl(composite);
	}

	protected void calcul_fdr(Button widget) throws MSMSException {
		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()
				&& (database_list.getSelectionCount() > 0 || !database_list
						.isEnabled())) {
			calculation_peptide_fdr calc_pep;
			calculation_protein_fdr calc_prot;
			// Si database non grise
			if (database_list.isEnabled()) {
				String[] databases = database_list.getSelection();
				calc_pep = new calculation_peptide_fdr(conf, databases);
				calc_prot = new calculation_protein_fdr(conf, databases);
			} else {
				calc_pep = new calculation_peptide_fdr(conf);
				calc_prot = new calculation_protein_fdr(conf);
			}

			int index = identif_select.getSelectionIndex();
			calc_pep.execute(XtandemPipelineSession.getInstance()
					.getIdentificationList().get(index).get_protein_liste());
			calc_prot.execute(XtandemPipelineSession.getInstance()
					.getIdentificationList().get(index).get_protein_liste());
			((Text) widget.getData("peptide")).setText(""
					+ calc_pep.get_peptide_fdr());
			((Text) widget.getData("protein")).setText(""
					+ calc_prot.get_protein_fdr());
		}
	}

	private void add_expand_bar_deltamh(ExpandBar bar) {
		// TODO Auto-generated method stub
		Composite composite = new Composite(bar, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		composite.setLayout(layout);

		Combo da_ppm = new Combo(composite, SWT.DROP_DOWN | SWT.READ_ONLY);
		da_ppm.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		da_ppm.add("ppm");
		da_ppm.add("Da");
		da_ppm.select(0);
		Label l1 = new Label(composite, SWT.NONE);
		l1.setText("Mass type");
		l1.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t1 = new Text(composite, SWT.SINGLE);
		t1.setText("NA");
		GridData col = new GridData(SWT.FILL, SWT.CENTER, false, false);
		col.horizontalAlignment = GridData.FILL;
		col.grabExcessHorizontalSpace = true;
		// col.minimumWidth = 70;
		t1.setLayoutData(col);
		Label l2 = new Label(composite, SWT.NONE);
		l2.setText("Mean");
		l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t2 = new Text(composite, SWT.SINGLE);
		t2.setText("NA");
		t2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l3 = new Label(composite, SWT.NONE);
		l3.setText("Median");
		l3.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t4 = new Text(composite, SWT.SINGLE);
		t4.setText("NA");
		t4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l4 = new Label(composite, SWT.NONE);
		l4.setText("Standard deviation");
		l4.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Button start = new Button(composite, SWT.PUSH);
		start.setText("Compute");
		start.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				try {
					Main_Windows.this.calcul_mass_deviation((Button) e.widget);
				} catch (MSMSException e1) {
					logger.error(e1.getMessage());
					e1.printStackTrace();
				}
			}
		});
		start.setData("type", da_ppm);
		start.setData("mean", t1);
		start.setData("median", t2);
		start.setData("sd", t4);
		buttons.add(start);

		ExpandItem item0 = new ExpandItem(bar, SWT.NONE, 0);
		item0.setText("Mass precision");
		item0.setHeight(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		item0.setControl(composite);
	}

	private void add_expand_bar_filter(ExpandBar bar) {
		// TODO Auto-generated method stub
		Composite composite = new Composite(bar, SWT.NONE);
		GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		composite.setLayout(layout);

		Text t1 = new Text(composite, SWT.SINGLE);
		t1.setText("" + conf.get_peptide_evalue());

		GridData col = new GridData(SWT.FILL, SWT.CENTER, false, false);
		col.horizontalAlignment = GridData.FILL;
		col.grabExcessHorizontalSpace = true;
		// col.minimumWidth = 70;
		t1.setLayoutData(col);
		Label l2 = new Label(composite, SWT.NONE);
		l2.setText("Peptide evalue");
		l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t1minPepProhet = new Text(composite, SWT.SINGLE);
		t1minPepProhet.setText("" + conf.getMinimumPeptideProphetProbability());
		t1minPepProhet.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false,
				false));
		Label l2minPepProhet = new Label(composite, SWT.NONE);
		l2minPepProhet.setText("Minimum peptide prophet probability");
		l2minPepProhet.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t1minPepInterProhet = new Text(composite, SWT.SINGLE);
		t1minPepInterProhet.setText(""
				+ conf.getMinimumPeptideInterProphetProbability());
		t1minPepInterProhet.setLayoutData(new GridData(SWT.FILL, SWT.CENTER,
				false, false));
		Label l2minPepInterProhet = new Label(composite, SWT.NONE);
		l2minPepInterProhet
				.setText("Minimum peptide inter prophet probability");
		l2minPepInterProhet
				.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t2 = new Text(composite, SWT.SINGLE);
		t2.setText("" + conf.get_peptide_number());
		t2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l3 = new Label(composite, SWT.NONE);
		l3.setText("Number of peptides per protein");
		l3.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t4 = new Text(composite, SWT.SINGLE);
		t4.setText("" + conf.get_protein_evalue());
		t4.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l4 = new Label(composite, SWT.NONE);
		l4.setText("Protein evalue (log)");
		l4.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		this.filter_button = new Button(composite, SWT.PUSH);
		filter_button.setText("Filter");
		filter_button.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Main_Windows.this.filter_results_to((Button) e.widget);
			}
		});
		filter_button.setData("peptide", t1);
		filter_button.setData("minimumPeptideProphetProbability",
				t1minPepProhet);
		filter_button.setData("minimumPeptideInterProphetProbability",
				t1minPepInterProhet);
		filter_button.setData("pepnumber", t2);
		filter_button.setData("protein", t4);
		buttons.add(filter_button);

		ExpandItem item0 = new ExpandItem(bar, SWT.NONE, 0);
		item0.setText("Filter identification results");
		item0.setHeight(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT).y);
		item0.setControl(composite);
	}

	private void update_filter_param() {
		((Text) filter_button.getData("peptide")).setText(""
				+ XtandemPipelineSession.getInstance().getConfig()
						.get_peptide_evalue());
		((Text) filter_button.getData("pepnumber")).setText(""
				+ XtandemPipelineSession.getInstance().getConfig()
						.get_peptide_number());
		((Text) filter_button.getData("protein")).setText(""
				+ XtandemPipelineSession.getInstance().getConfig()
						.get_protein_evalue());
		((Text) filter_button.getData("minimumPeptideProphetProbability"))
				.setText(""
						+ XtandemPipelineSession.getInstance().getConfig()
								.getMinimumPeptideProphetProbability());
		((Text) filter_button.getData("minimumPeptideInterProphetProbability"))
				.setText(""
						+ XtandemPipelineSession.getInstance().getConfig()
								.getMinimumPeptideInterProphetProbability());
	}

	protected void filter_results_to(Button widget) {
		try {
			if (!XtandemPipelineSession.getInstance().getDataTypeBase()
					.getIs_active())
				throw new StopException("");
			float fl1;
			float fl2;
			int it;
			try {
				fl1 = Float.valueOf(
						((Text) widget.getData("protein")).getText())
						.floatValue();
			} catch (Exception e) {
				throw new MSMSException(
						"Log(E value) protein must be like '-8'");
			}
			try {
				fl2 = Float.valueOf(
						((Text) widget.getData("peptide")).getText())
						.floatValue();
			} catch (Exception e) {
				throw new MSMSException("Peptide E value must be like '0.05'");
			}
			try {
				it = Integer.parseInt(((Text) widget.getData("pepnumber"))
						.getText());
			} catch (Exception e) {
				throw new MSMSException("Peptide number must be a number '2'");
			}

			conf.setMinimumPeptideProphetProbability(((Text) widget
					.getData("minimumPeptideProphetProbability")).getText());
			conf.setMinimumPeptideInterProphetProbability(((Text) widget
					.getData("minimumPeptideInterProphetProbability"))
					.getText());

			conf.set_protein_evalue(fl1);
			conf.set_peptide_evalue(fl2);
			conf.set_peptide_number(it);

			XtpProcess p = new reinitialise_result_filter_and_create_grouping(
					conf, XtandemPipelineSession.getInstance()
							.getIdentificationList()
							.get(identif_select.getSelectionIndex()),
					XtandemPipelineSession.getInstance().getDataTypeBase());
			new Running(this, p, false);

			if (p.isStop())
				throw new StopException("");
			if (p.getError() != null)
				throw p.getError();

			this.view_result();

			this.modificationsApplied();

		} catch (StopException e) {
			// TODO Auto-generated catch block
			logger.debug("stop");
		} catch (MSMSException e) {
			MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
			messageBox.setMessage(e.getMessage());
			messageBox.open();
		} catch (Exception e) {
			this.view_error(e, "Problems trying to filter results :");
		}
	}

	protected void calcul_mass_deviation(Button widget) throws MSMSException {
		boolean type = false;
		if (((Combo) widget.getData("type")).getSelectionIndex() == 0)
			type = true;
		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			calculation_peptide_mh_deviation calc = new calculation_peptide_mh_deviation(
					type);
			int index = identif_select.getSelectionIndex();
			calc.execute(XtandemPipelineSession.getInstance()
					.getIdentificationList().get(index).get_peptide_liste());
			((Text) widget.getData("mean")).setText(Utils.formatDecimal(calc
					.get_mean_deviation()));
			((Text) widget.getData("median")).setText(Utils.formatDecimal(calc
					.get_mediane_deviation()));
			((Text) widget.getData("sd")).setText(Utils.formatDecimal(calc
					.get_standart_deviation()));
		}
	}

	private void set_result_view() {
		// TODO Auto-generated method stub
		Group result = new Group(shell, SWT.SHADOW_IN);
		result.setText("Identification Results");
		result.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		GridLayout layout = new GridLayout(2, false);
		layout.marginLeft = layout.marginTop = layout.marginRight = layout.marginBottom = 10;
		layout.verticalSpacing = 10;
		result.setLayout(layout);

		Button view = new Button(result, SWT.PUSH);
		view.setText("View");
		view.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Main_Windows.this.view_result();
			}
		});
		buttons.add(view);
		identif_select = new Combo(result, SWT.DROP_DOWN | SWT.READ_ONLY);
		identif_select.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t1 = new Text(result, SWT.SINGLE | SWT.READ_ONLY);
		t1.setText("0");
		t1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l2 = new Label(result, SWT.NONE);
		l2.setText("Number of samples");
		l2.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t2 = new Text(result, SWT.SINGLE | SWT.READ_ONLY);
		t2.setText("0");
		t2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l3 = new Label(result, SWT.NONE);
		l3.setText("Number of groups");
		l3.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		Text t3 = new Text(result, SWT.SINGLE | SWT.READ_ONLY);
		t3.setText("0");
		t3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false));
		Label l4 = new Label(result, SWT.NONE);
		l4.setText("Number of subgroups");
		l4.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));

		identif_select.setData("button", view);
		identif_select.setData("sample", t1);
		identif_select.setData("group", t2);
		identif_select.setData("subgroup", t3);

	}

	private void set_menu() {
		// Create the bar menu
		Menu menuBar = new Menu(shell, SWT.BAR);

		// Create the File item's dropdown menu
		Menu fileMenu = new Menu(menuBar);
		Menu windowmenu = new Menu(menuBar);
		Menu load = new Menu(fileMenu);
		Menu xtandem = new Menu(fileMenu);
		Menu aboutmenu = new Menu(menuBar);
		Menu toolsmenu = new Menu(menuBar);
		Menu pep_list = new Menu(windowmenu);
		Menu prot_list = new Menu(windowmenu);
		Menu pep_det = new Menu(windowmenu);
		Menu prot_det = new Menu(windowmenu);

		// create list of windows menu
		MenuItem[] windows_list = new MenuItem[4];

		// Create all the items in the bar menu
		MenuItem fileItem = new MenuItem(menuBar, SWT.CASCADE);
		fileItem.setText("File");
		fileItem.setMenu(fileMenu);

		MenuItem item = new MenuItem(fileMenu, SWT.CASCADE);
		item.setText("X!Tandem");
		item.setMenu(xtandem);

		MenuItem xtandemitem = new MenuItem(xtandem, SWT.NONE);
		xtandemitem.setText("Analysis");
		new MenuItem(fileMenu, SWT.SEPARATOR);

		// Create all the items in the File dropdown menu
		MenuItem newItem = new MenuItem(fileMenu, SWT.CASCADE);
		newItem.setText("Load result");
		newItem.setMenu(load);

		MenuItem indiv = new MenuItem(load, SWT.NONE);
		indiv.setText("Individual");
		MenuItem combi = new MenuItem(load, SWT.NONE);
		combi.setText("Combine");
		MenuItem phospho = new MenuItem(load, SWT.NONE);
		phospho.setText("Phosphopeptide");

		MenuItem projetitem = new MenuItem(fileMenu, SWT.NONE);
		projetitem.setText("Load project");

		menuItemSaveProject = new MenuItem(fileMenu, SWT.NONE);
		menuItemSaveProject.setText("Save project");
		menuItemSaveProject.setEnabled(false);
		items.add(menuItemSaveProject);

		// this.menuItemSaveProject = projetitem;

		menuItemSaveProjectAs = new MenuItem(fileMenu, SWT.NONE);
		menuItemSaveProjectAs.setText("Save project as");
		menuItemSaveProjectAs.setEnabled(false);
		items.add(menuItemSaveProjectAs);

		MenuItem exportitem = new MenuItem(fileMenu, SWT.NONE);
		exportitem.setText("Export");
		exportitem.setEnabled(false);
		items.add(exportitem);

		new MenuItem(fileMenu, SWT.SEPARATOR);
		MenuItem closeItem = new MenuItem(fileMenu, SWT.NONE);
		closeItem.setText("Exit");
		shell.setMenuBar(menuBar);

		MenuItem tools = new MenuItem(menuBar, SWT.CASCADE);
		tools.setText("Options");
		tools.setMenu(toolsmenu);

		MenuItem toolitem = new MenuItem(toolsmenu, SWT.NONE);
		toolitem.setText("Path preference");
		toolitem = new MenuItem(toolsmenu, SWT.NONE);
		toolitem.setText("X!Tandem preset");

		MenuItem windowItem = new MenuItem(menuBar, SWT.CASCADE);
		windowItem.setText("Windows");
		windowItem.setMenu(windowmenu);

		windows_list[0] = new MenuItem(windowmenu, SWT.CASCADE);
		windows_list[0].setText("Protein list");
		windows_list[0].setData("windows", protein_list);
		windows_list[0].setMenu(prot_list);
		this.protein_list.set_menu_item(windows_list[0]);
		windows_list[1] = new MenuItem(windowmenu, SWT.CASCADE);
		windows_list[1].setText("Protein details");
		windows_list[1].setData("windows", protein_details);
		windows_list[1].setMenu(prot_det);
		this.protein_details.set_menu_item(windows_list[1]);
		windows_list[2] = new MenuItem(windowmenu, SWT.CASCADE);
		windows_list[2].setText("Peptide list");
		windows_list[2].setData("windows", peptide_list);
		windows_list[2].setMenu(pep_list);
		this.peptide_list.set_menu_item(windows_list[2]);
		windows_list[3] = new MenuItem(windowmenu, SWT.CASCADE);
		windows_list[3].setText("Peptide details");
		windows_list[3].setData("windows", peptide_details);
		windows_list[3].setMenu(pep_det);
		this.peptide_details.set_menu_item(windows_list[3]);

		for (MenuItem itema : windows_list) {
			MenuItem open = new MenuItem(itema.getMenu(), SWT.CHECK);
			open.setText("Open window");
			MenuItem hide = new MenuItem(itema.getMenu(), SWT.NONE);
			hide.setText("Hide window");
			hide.setEnabled(false);
			MenuItem clos = new MenuItem(itema.getMenu(), SWT.NONE);
			clos.setText("Close window");
			clos.setEnabled(false);
		}

		new MenuItem(windowmenu, SWT.SEPARATOR);
		MenuItem preference = new MenuItem(windowmenu, SWT.NONE);
		preference.setText("Preferences");
		preference.setEnabled(false);

		MenuItem about = new MenuItem(menuBar, SWT.CASCADE);
		about.setText("Help");
		about.setMenu(aboutmenu);
		MenuItem aboutitem = new MenuItem(aboutmenu, SWT.NONE);
		aboutitem.setText("About");
		aboutitem = new MenuItem(aboutmenu, SWT.NONE);
		aboutitem.setText("Documentation");
		aboutitem = new MenuItem(aboutmenu, SWT.NONE);
		aboutitem.setText("Warning");
		aboutitem = new MenuItem(aboutmenu, SWT.NONE);
		aboutitem.setText("News");
	}

	public Display getDisplay() {
		return display;
	}

	public void setDisplay(Display display) {
		this.display = display;
	}

	public Image[] getImages() {
		return images;
	}

	public void setImages(Image[] images) {
		this.images = images;
	}

	private void update_identification_result() {
		logger.debug("Number of identification results : "
				+ XtandemPipelineSession.getInstance().getIdentificationList()
						.size());
		identif_select.removeAll();
		logger.debug("datatype="
				+ XtandemPipelineSession.getInstance().getDataTypeBase());
		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			if (XtandemPipelineSession.getInstance().getDataTypeBase()
					.getIs_invididual()) {
				for (Identification ident : XtandemPipelineSession
						.getInstance().getIdentificationList()) {
					identif_select.add(ident.getMsRunSet().getMsRunList()
							.get(0).getSampleName());
				}
			} else if (!XtandemPipelineSession.getInstance().getDataTypeBase()
					.getIs_phosphopeptide()) {
				identif_select.add("Combine result");
			} else {
				identif_select.add("Phosphopeptide result");
			}
			identif_select.select(0);
			identif_select.update();
			this.view_result();

			// on met à jour les valeurs de filtrage
			this.update_filter_param();

			// on degrise les menus
			for (Button but : buttons)
				but.setEnabled(true);
			for (MenuItem it : items)
				it.setEnabled(true);

		}

	}

	private void identificationResultsCleared() {
		logger.debug("identificationResultsCleared begin ");
		XtandemPipelineSession.getInstance().forgetExistingXpipFile();

		menuItemSaveProject.setEnabled(false);
		this.menuItemSaveProjectAs.setEnabled(false);

		refreshIdentificationResultsInfoFrame();

	}

	private void newIdentificationsLoaded() {
		logger.debug("newIdentificationsLoaded begin ");

		XtandemPipelineSession.getInstance().forgetExistingXpipFile();

		menuItemSaveProject.setEnabled(true);
		this.menuItemSaveProjectAs.setEnabled(true);

	}

	public void modificationsApplied() {
		logger.debug("modificationsApplied begin");
		// tell xpip file (if any) that results are modified
		XpipFile xpipFile = XtandemPipelineSession.getInstance()
				.getCurrentXpipFile();
		if (xpipFile != null) {
			xpipFile.modified();
			menuItemSaveProject.setEnabled(true);
		} else {
			menuItemSaveProject.setEnabled(false);
			this.menuItemSaveProjectAs.setEnabled(true);
		}
		// refresh grouping info display
		this.refreshIdentificationResultsInfoFrame();
	}

	private void xpipFileLoaded() {
		logger.debug("xpipFileLoaded begin ");
		// there is a new project loaded from a file :
		// update menuitem
		XpipFile xpipFile = XtandemPipelineSession.getInstance()
				.getCurrentXpipFile();
		if (xpipFile != null) {
			if (xpipFile.isModified()) {
				menuItemSaveProject.setEnabled(true);
			} else {
				menuItemSaveProject.setEnabled(false);
			}
		} else {
			menuItemSaveProject.setEnabled(false);
			this.menuItemSaveProjectAs.setEnabled(true);
		}
	}

	private void xpipFileSaved() {
		logger.debug("xpipFileSaved begin ");
		this.xpipFileLoaded();
	}

	protected boolean view_result() {
		if (!XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			return (false);
		}

		this.refreshIdentificationResultsInfoFrame();

		// on mets à jour la liste des bases de données
		database_list.removeAll();
		int index = identif_select.getSelectionIndex();

		for (Database url : XtandemPipelineSession.getInstance()
				.getIdentificationList().get(index).get_database_list())
			database_list.add(url.getDatabasePath());

		// envoyer les signaux aux autres fenetres
		this.protein_list.load_data(XtandemPipelineSession.getInstance()
				.getIdentificationList().get(index), XtandemPipelineSession
				.getInstance().getDataTypeBase());
		return (true);
	}

	private void refreshIdentificationResultsInfoFrame() {
		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			int index = identif_select.getSelectionIndex();
			logger.info("Selected index of identificationList:" + index);
			Text t = (Text) identif_select.getData("sample");
			t.setText(""
					+ XtandemPipelineSession.getInstance()
							.getIdentificationList().get(index).getMsRunSet()
							.size());
			// shell.setText(identif_select.getItem(index));
			Text t2 = (Text) identif_select.getData("group");
			t2.setText(""
					+ XtandemPipelineSession.getInstance()
							.getIdentificationList().get(index).getGrouping()
							.getGroupList().size());

			Text t3 = (Text) identif_select.getData("subgroup");
			t3.setText(""
					+ XtandemPipelineSession.getInstance()
							.getIdentificationList().get(index).getGrouping()
							.getNumberOfSubgroups());
			t.update();
			t2.update();
			t3.update();
		} else {
			// on vide les fenetres
			identif_select.removeAll();
			database_list.removeAll();
			Text t = (Text) identif_select.getData("sample");
			t.setText("" + 0);
			Text t2 = (Text) identif_select.getData("group");
			t2.setText("" + 0);

			Text t3 = (Text) identif_select.getData("subgroup");
			t3.setText("" + 0);
		}
	}

	protected boolean reset_view() {
		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			XpipFile xpipFile = XtandemPipelineSession.getInstance()
					.getCurrentXpipFile();
			boolean warning = true;
			if (xpipFile != null) {
				if (xpipFile.isModified() == false) {
					warning = false;
				}
			}
			if (warning) {
				MessageBox messageBox = new MessageBox(shell, SWT.YES | SWT.NO);
				messageBox
						.setMessage("You are about to start a new project."
								+ " All unsaved modifications will be lost.\n Proceed with a new project ?");
				int response = messageBox.open();
				if (response == SWT.NO) {
					return (false);
				}
			}
		}

		// on grise les menus
		for (Button but : buttons)
			but.setEnabled(false);
		for (MenuItem it : items)
			it.setEnabled(false);

		// on les fenetres pour ce remettre à zero
		Shell[] ses = display.getShells();
		for (Shell se : ses) {
			if (se.getData("menuitem") != null) {
				se.dispose();
			}
		}

		// on met à zero la session
		XtandemPipelineSession.getInstance().clearIdentificationResult();

		this.identificationResultsCleared();
		return (true);
	}

	public void view_protein(Identification ident, SubGroup sg,
			HashSampleScanSetProt hashProt) {
		this.peptide_list.load_data(ident, hashProt, XtandemPipelineSession
				.getInstance().getDataTypeBase());
		this.protein_details.view_protein_details(hashProt);
	}

	public void view_peptide(Peptide pep) {
		peptide_details.draw_spectra_to(pep);

	}

	protected void open_closed_shell(MenuItem menu) {
		if (menu.getSelection()) {
			window_base base = (window_base) menu.getData("windows");
			base.open_windows();

		} else {
			Shell[] ses = display.getShells();
			for (Shell se : ses) {
				if ((se.getData("menuitem") != null)
						&& (se.getData("menuitem").equals(menu)))
					se.dispose();
			}
		}
	}

	protected void open_shell(MenuItem menu) {
		Shell[] ses = display.getShells();
		for (Shell se : ses) {
			if ((se.getData("menuitem") != null)
					&& (se.getData("menuitem").equals(menu)))
				se.setMinimized(false);
		}

		window_base base = (window_base) menu.getData("windows");
		if (base.getShell() == null || base.getShell().isDisposed())
			base.open_windows();
		else {
			base.getShell().setMinimized(false);
			base.getShell().setActive();
		}
		menu.getMenu().getItem(0).setSelection(true);
		menu.getMenu().getItem(1).setEnabled(true);
		menu.getMenu().getItem(2).setEnabled(true);
	}

	protected void close_shell(MenuItem menu) {
		Shell[] ses = display.getShells();
		for (Shell se : ses) {
			if ((se.getData("menuitem") != null)
					&& (se.getData("menuitem").equals(menu)))
				se.dispose();
		}
	}

	protected void hide_shell(MenuItem menu) {
		Shell[] ses = display.getShells();
		for (Shell se : ses) {
			if ((se.getData("menuitem") != null)
					&& (se.getData("menuitem").equals(menu)))
				se.setMinimized(true);
		}
	}

	protected void export_identification_results() {
		try {
			DataType_Base datatype = XtandemPipelineSession.getInstance()
					.getDataTypeBase();
			if (!datatype.getIs_active())
				throw new StopException("");

			XpipFile xpipFile = XtandemPipelineSession.getInstance()
					.getCurrentXpipFile();
			if (xpipFile == null) {
				MessageBox messageBox = new MessageBox(shell, SWT.YES | SWT.NO);
				messageBox
						.setMessage("Your project is not saved. Please consider saving your project into an XPIP file before exporting results.\n Continue?");
				int response = messageBox.open();
				if (response == SWT.NO) {
					return;
				}
			} else {
				if (xpipFile.isModified()) {
					MessageBox messageBox = new MessageBox(shell, SWT.YES
							| SWT.NO);
					messageBox
							.setMessage("There are unsaved modifications in your project. Please consider saving your project before exporting results.\n Continue?");
					int response = messageBox.open();
					if (response == SWT.NO) {
						return;
					}
				}
			}

			// on recree un nouveau printer sinon erreur si déjà processer
			datatype.newPrint();

			Export_Windows export = new Export_Windows(this);
			File S_out = export.getFileOut();
			if (export.isCancelled()) {
				throw new StopException("Stopping");
			}

			logger.debug("new Running(this, datatype.getPrint(), false)");
			new Running(this, datatype.getPrint(), false);
			logger.debug("after new Running(this, datatype.getPrint(), false)");

			if (datatype.getPrint().isStop()) {
				throw new StopException("");
			}
			if (datatype.getPrint().getError() != null) {
				throw datatype.getPrint().getError();
			}

			MessageBox messageBox = new MessageBox(shell, SWT.OK);
			messageBox.setMessage("Export results save as :\n"
					+ S_out.getAbsolutePath() + "*");
			messageBox.open();
		} catch (StopException e) {
			// TODO Auto-generated catch block
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems trying to export results :");
		}
	}

	protected void readIdentificationsResults(DataType_Base type) {
		try {
			// on met à jour le type de datatype
			XtandemPipelineSession.getInstance().setDataTypeBase(type);
			// configuration des paramètres de filtrage
			Filter_Windows filter = new Filter_Windows(this);
			if (filter.isCancelled())
				throw new StopException("Stopping");

			new Running(this, type.getPro(), false);

			if (type.getPro().isStop())
				throw new StopException("Stopping");
			if (type.getPro().getError() != null)
				throw type.getPro().getError();

			// identifications = type.getPro().getIdentifications();
			if (XtandemPipelineSession.getInstance().getIdentificationList()
					.size() == 0) {
				throw new MSMSException("Not any valid identification");
			}
			this.update_identification_result();
			this.newIdentificationsLoaded();

		} catch (StopException e) {
			// TODO Auto-generated catch block
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems trying to load results :");
		}
	}

	protected void launchXtandemAnalysis() {
		try {
			logger.debug("launchXtandemAnalysis begin");
			if (!conf.getXtandem_software_installed())
				throw new MSMSException(
						"X!Tandem executable is not installed. Please see :"
								+ "\n\tOptions->Path configuration\n\tAbout->Documentation");
			xtandem_analysis xtande;
			if (conf.getCondorSubmitCommand() == null) {
				xtande = new xtandem_analysis(conf);
			} else {
				xtande = new XtandemCondorAnalysis(conf);
			}
			logger.debug(" new Xtandem_Analysis_Param begin");
			Xtandem_Analysis_Param params = new Xtandem_Analysis_Param(this,
					conf, xtande);
			if (params.getCancelled())
				throw new StopException("");
			logger.debug(" new Running(this, xtande, true) begin");
			new Running(this, xtande, true);

			if (xtande.isStop())
				throw new StopException("");
			if (xtande.getError() != null)
				throw xtande.getError();

			MessageBox messageBox = new MessageBox(shell, SWT.OK);
			messageBox
					.setMessage("Analysis finished, the results are saved in the folder :\n"
							+ xtande.getSave_dir().getAbsolutePath() + "*");
			messageBox.open();

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during X!Tandem analysis :");
		}
	}

	public void view_error(Exception e, String text_error) {
		logger.error(text_error);
		logger.error(e.getMessage());
		StackTraceElement[] f = e.getStackTrace();
		String error;
		if (e.getClass().equals(MSMSException.class)) {
			error = e.getMessage();
		} else {
			error = e.getMessage() + "at : \n";
			for (StackTraceElement trace : f) {
				error += trace.toString() + "\n";
			}
		}
		MessageBox messageBox = new MessageBox(shell, SWT.ERROR);
		messageBox.setMessage(text_error + "\n" + error);
		messageBox.open();
	}

	public void file_dialog_load_current_project() {
		try {
			XpipFile xpip = this.get_file_project("Load project", SWT.OPEN);
			// XpipFile out = this.get_file_project("Load project", SWT.OPEN);

			// XtandemPipelineSession.getInstance().getInstance().getCurrentXpipFile().read(conf);
			load_result_and_create_grouping loading = new load_result_and_create_grouping(
					conf, xpip);
			this.loadCurrentProject(loading);
			XtandemPipelineSession.getInstance().setCurrentXpipFile(xpip);

			this.xpipFileLoaded();

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during loading project :");
		}
	}

	private void loadCurrentProject(load_result_and_create_grouping loading) {
		try {
			logger.debug("loadCurrentProject begin ");
			new Running(this, loading, false);

			if (loading.isStop())
				throw new StopException("");
			if (loading.getError() != null)
				throw loading.getError();

			this.update_identification_result();

			// Si la version nest pas indiqué dans le fichier on indique un
			// message à l'utilisateur
			if (loading.getVersion() == null) {
				MessageBox messageBox = new MessageBox(shell, SWT.ICON_WARNING
						| SWT.OK);
				messageBox
						.setMessage("This xtandempipeline project was an old file version."
								+ "\n"
								+ "To share benefits of the new grouping algorithm, we recommand to refilter the results.");
				messageBox.open();
			}

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during loading project :");
		}

	}

	protected void saveCurrentProject() {
		try {
			if (!XtandemPipelineSession.getInstance().getDataTypeBase()
					.getIs_active())
				throw new StopException("There is nothing to save");

			XpipFile xpipFile = XtandemPipelineSession.getInstance()
					.getCurrentXpipFile();
			if (xpipFile == null) {
				// throw new StopException("There is no current XPIP file");
				saveCurrentProjectAs();
			} else {

				save_result_and_filtering save = new save_result_and_filtering(
						conf, xpipFile);
				new Running(this, save, false);

				if (save.isStop())
					throw new StopException("");
				if (save.getError() != null)
					throw save.getError();

				this.xpipFileSaved();
			}
			// XtandemPipelineSession.getInstance().setCurrentXpipFile(out);

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during saving project :");
		}
	}

	protected void saveCurrentProjectAs() {
		try {
			if (!XtandemPipelineSession.getInstance().getDataTypeBase()
					.getIs_active())
				throw new StopException("");

			XpipFile out = this.get_file_project("Save current project as",
					SWT.SAVE);
			save_result_and_filtering save = new save_result_and_filtering(
					conf, out);
			new Running(this, save, false);

			if (save.isStop())
				throw new StopException("");
			if (save.getError() != null)
				throw save.getError();

			XtandemPipelineSession.getInstance().setCurrentXpipFile(out);
			this.xpipFileSaved();

		} catch (StopException e) {
			logger.debug("stop");
		} catch (Exception e) {
			this.view_error(e, "Problems during saving project :");
		}
	}

	private XpipFile get_file_project(String information, int type)
			throws StopException {
		FileDialog dlg = new FileDialog(shell, type | SWT.SINGLE);
		dlg.setFilterPath(conf.getXmlresult().getPath());
		dlg.setText(information);
		dlg.setFilterExtensions(new String[] { "*.xpip", "*.*" });
		dlg.setFilterNames(new String[] { "X!Tandem pipeline project",
				"All files" });
		String fileName = dlg.open();
		if (fileName == null)
			throw new StopException("No project file selected");
		fileName = fileName.replaceFirst("\\.xpip$", "");
		XpipFile xpip = new XpipFile(fileName + ".xpip");
		return (xpip);
	}

	protected boolean wantToCloseXtandemPipeline() {
		logger.debug("wantToCloseXtandemPipeline begin");

		if (XtandemPipelineSession.getInstance().getDataTypeBase()
				.getIs_active()) {
			XpipFile xpipFile = XtandemPipelineSession.getInstance()
					.getCurrentXpipFile();
			if (xpipFile == null) {
				MessageBox messageBox = new MessageBox(shell, SWT.YES | SWT.NO);
				messageBox
						.setMessage("Your project is not saved. Please consider saving your project into an XPIP file before exiting.\n Continue exiting ?");
				int response = messageBox.open();
				if (response == SWT.NO) {
					return false;
				}
			} else {
				if (xpipFile.isModified()) {
					MessageBox messageBox = new MessageBox(shell, SWT.YES
							| SWT.NO);
					messageBox
							.setMessage("There are unsaved modifications in your project. Please consider saving your project before exiting.\n Continue exiting ?");
					int response = messageBox.open();
					if (response == SWT.NO) {
						return false;
					}
				}
			}
		}
		return true;
	}

}

class ResultSelectionListener implements Listener {
	private static final Logger logger = Logger
			.getLogger(ResultSelectionListener.class);
	Main_Windows mainWindows;

	public ResultSelectionListener(Main_Windows name) {
		this.mainWindows = name;
	}

	public void handleEvent(Event e) {
		// System.out.println("Event: [" + e.toString() + "]");
		// System.out.println(e.widget.getStyle());
		// Menu Check
		try {
			if (e.widget.getStyle() == 32) {
				// name.open_closed_shell((MenuItem) e.widget);
				MenuItem check = (MenuItem) e.widget;
				if (check.getText().equals("Open window"))
					mainWindows.open_shell(check.getParent().getParentItem());
			}
			// menu Item
			else if (e.widget.getStyle() == 8) {
				MenuItem it = (MenuItem) e.widget;
				if (it.getText().equals("Exit")) {
					if (mainWindows.wantToCloseXtandemPipeline()) {
						mainWindows.shell.dispose();
					}
				} else if (it.getText().equals("Load project")) {
					if (mainWindows.reset_view())
						mainWindows.file_dialog_load_current_project();
				} else if (it == mainWindows.menuItemSaveProject) {
					logger.debug("menuItemSaveProject clicked");
					mainWindows.saveCurrentProject();
				} else if (it == mainWindows.menuItemSaveProjectAs) {
					// } else if (it.getText().equals("Save project"))
					logger.debug("menuItemSaveProjectAs clicked");
					mainWindows.saveCurrentProjectAs();
				} else if (it.getText().equals("Export"))
					mainWindows.export_identification_results();
				else if (it.getText().equals("Analysis")) {
					logger.debug("it.getText().equals(\"Analysis\")");
					mainWindows.launchXtandemAnalysis();
				} else if (it.getText().equals("Combine")) {
					if (mainWindows.reset_view())
						mainWindows
								.readIdentificationsResults(new DataType_Combine(
										mainWindows.conf));
				} else if (it.getText().equals("Individual")) {
					if (mainWindows.reset_view())
						mainWindows
								.readIdentificationsResults(new DataType_individual(
										mainWindows.conf));
				} else if (it.getText().equals("Phosphopeptide")) {
					if (mainWindows.reset_view())
						mainWindows
								.readIdentificationsResults(new DataType_phosphopeptide(
										mainWindows.conf));
				} else if (it.getText().equals("Path preference"))
					new Path_configuration(mainWindows);
				else if (it.getText().equals("X!Tandem preset"))
					new Xtandem_preset(mainWindows, mainWindows.conf);
				else if (it.getText().equals("Documentation")) {
					String u = "http://pappso.inra.fr/downloads/xtandem_pipeline.pdf";
					if (Desktop.isDesktopSupported()) {
						// On récupère l'instance du desktop :
						Desktop desktop = Desktop.getDesktop();
						if (desktop.isSupported(Desktop.Action.BROWSE)) {
							// Et on lance l'application associé au protocole :
							try {
								desktop.browse(new URI(u));
							} catch (Exception error) {
								error.printStackTrace();
							}
						}
					}
				} else if (it.getText().equals("Warning")) {
					MessageBox messageBox = new MessageBox(mainWindows.shell,
							SWT.ICON_WARNING | SWT.OK);
					messageBox.setText("Warning");
					messageBox.setMessage(XtandemPipelineMain.warning);
					messageBox.open();
				} else if (it.getText().equals("About")) {
					MessageBox messageBox = new MessageBox(mainWindows.shell,
							SWT.ICON_QUESTION | SWT.OK);
					messageBox.setText("About");
					messageBox.setMessage(XtandemPipelineMain.about);
					messageBox.open();
				} else if (it.getText().equals("News")) {
					MessageBox messageBox = new MessageBox(mainWindows.shell,
							SWT.OK);
					messageBox.setText("News on version : \""
							+ XtandemPipelineMain.versionName + "\"");
					messageBox.setMessage(XtandemPipelineMain.news);
					messageBox.open();
				} else if (it.getText().equals("Hide window")) {
					mainWindows.hide_shell(it.getParent().getParentItem());
				} else if (it.getText().equals("Close window")) {
					mainWindows.close_shell(it.getParent().getParentItem());
				}
			}

		} catch (Exception e1) {
			e1.printStackTrace();
			// throw new SAXException(message);
		}
	}

}