package fr.inra.pappso.xtandempipeline.ui.results;

import java.util.ArrayList;
import java.util.Vector;

import fr.inra.pappso.xtandempipeline.MsException.MSMSException;
import fr.inra.pappso.xtandempipeline.MsException.StopException;
import fr.inra.pappso.xtandempipeline.class_msms.Identification;
import fr.inra.pappso.xtandempipeline.class_msms.Match;
import fr.inra.pappso.xtandempipeline.class_msms.MapDb.Peptide;
import fr.inra.pappso.xtandempipeline.filter_print.swt_table_protein;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScan;
import fr.inra.pappso.xtandempipeline.grouping.HashSampleScanSetProt;
import fr.inra.pappso.xtandempipeline.grouping.SubGroup;
import fr.inra.pappso.xtandempipeline.grouping.interfaces.GrpPeptide;
import fr.inra.pappso.xtandempipeline.thread.DataType_Base;
import fr.inra.pappso.xtandempipeline.thread.XtpProcess;
import fr.inra.pappso.xtandempipeline.thread.filtered_and_create_grouping;
import fr.inra.pappso.xtandempipeline.ui.swt.Running;

import org.apache.log4j.Logger;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;

public class Protein_List_item extends CTabItem {

	private Button cancel;

	private Vector<Button> buttons;

	protected window_base window;

	protected Identification ident;

	protected DataType_Base data_type;

	private Table table;

	private Label description;

	private static final Logger logger = Logger
			.getLogger(Protein_List_item.class);

	public Protein_List_item(window_base parent, int posi) {
		super(parent.getFolder(), SWT.CLOSE, posi);
		this.window = parent;
		buttons = new Vector<Button>();
		this.data_type = new DataType_Base(window.getConf());
		// on ajoute le contenu
		this.setControl(this.create_composite());
		for (Button but : buttons)
			but.setEnabled(false);
	}

	@Override
	protected void finalize() throws Throwable {
		logger.debug("finalize Protein_List_item");
	}

	private Composite create_composite() {
		Composite compo = new Composite(this.getParent(), SWT.None);
		GridLayout lay = new GridLayout(5, false);
		lay.verticalSpacing = 10;
		lay.horizontalSpacing = 5;
		compo.setLayout(lay);

		Text t = new Text(compo, SWT.SINGLE);
		t.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Button but = new Button(compo, SWT.PUSH);
		but.setText("Search");
		but.setData("text", t);
		but.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Text t = (Text) e.widget.getData("text");
				Protein_List_item.this.filter_data(t.getText());
				logger.debug("Search");
				Protein_List_item.this.cancel.setEnabled(true);
			}

		});
		this.buttons.add(but);
		this.cancel = new Button(compo, SWT.PUSH);
		cancel.setText("Cancel");
		cancel.setData("text", t);
		cancel.setEnabled(false);
		cancel.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Text t = (Text) e.widget.getData("text");
				t.setText("");
				Protein_List_item.this.filter_data(t.getText());
				logger.debug("Search cancelled");
				Protein_List_item.this.cancel.setEnabled(false);
			}

		});
		Label fill = new Label(compo, SWT.NONE);
		fill.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		Button but2 = new Button(compo, SWT.PUSH);
		but2.setText("Apply modifications");
		but2.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				logger.debug("remove");
				if (data_type.getIs_active()) {
					XtpProcess p = new filtered_and_create_grouping(window
							.getConf(), ident, data_type);
					new Running(window.getControlleur(), p, false);
					Protein_List_item.this.load_data(ident, data_type);
					window.getControlleur().modificationsApplied();
				}
			}

		});
		this.buttons.add(but2);
		table = new Table(compo, SWT.BORDER | SWT.CHECK | SWT.FULL_SELECTION);
		GridData data = new GridData(GridData.FILL_BOTH);
		data.horizontalSpan = 5;
		table.setLayoutData(data);
		table.setHeaderVisible(true);
		table.setLinesVisible(false);
		table.addSelectionListener(new SelectionAdapter() {
			private int action_time = 0;

			public void widgetSelected(SelectionEvent e) {
				logger.debug("selected");
				TableItem item = (TableItem) e.item;
				HashSampleScanSetProt m = (HashSampleScanSetProt) item
						.getData("match");
				SubGroup sg = (SubGroup) item.getData("subgroup");
				if (e.detail == SWT.CHECK) {
					try {
						Protein_List_item.this.changeProtValidity(sg,
								item.getChecked());
					} catch (MSMSException e1) {
						logger.error(e1.getMessage());
						e1.printStackTrace();
					}
					logger.debug("Change prot validity");
					action_time = e.time;
				} else if (e.time != action_time) {
					logger.debug("view prot");
					window.getControlleur().view_protein(ident, sg, m);
				}
			}

		});
		description = new Label(compo, SWT.NONE);
		description.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true,
				false, 5, 1));
		return compo;
	}

	protected void changeProtValidity(SubGroup sg, boolean change)
			throws MSMSException {
		// On change la validité des peptides associé à chaque protéine du
		// sous-groupe
		for (TableItem it : table.getItems()) {
			if (it.getData("subgroup").equals(sg)) {
				it.setChecked(change);
				HashSampleScanSetProt hp = (HashSampleScanSetProt) it
						.getData("match");
				if (change == false) {
					// unvalidate
					ArrayList<Peptide> peptidechecked = new ArrayList<Peptide>();
					for (HashSampleScan hash : hp) {
						for (GrpPeptide pepG : hp.getGrpProtMatch()
								.getGrpPeptidesSharingHashSampleScan(hash)) {
							Peptide pep = (Peptide) pepG;
							pep.setIsValid(false);
							peptidechecked.add(pep);
						}
					}
					it.setData("peptide_checked", peptidechecked);
				} else {
					// validate
					ArrayList<?> objectList = (ArrayList<?>) it
							.getData("peptide_checked");
					if (objectList != null) {
						for (Object objectPeptide : objectList) {
							((Peptide) objectPeptide).setIsValid(true);
						}
					}
				}
				// Si analyse non phospho, on peux aussi modifier l'etat du
				// match pour affichage
				if (!this.data_type.getIs_phosphopeptide()) {
					((Match) hp.getGrpProtMatch()).setIsvalidate(change);
				}
			}
		}
	}

	protected void filter_data(String text) {
		try {
			if (!data_type.getIs_active())
				throw new StopException("");
			swt_table_protein swt = new swt_table_protein(table, data_type);
			swt.setfilter(text);
			ident.view_result(swt);
		} catch (StopException e) {
			logger.debug("no data");
		} catch (Exception e) {
			window.view_error(e, "Problems with drawing protein list");
		}
	}

	public void load_data(Identification iden, DataType_Base datatype) {
		try {
			for (Button but : buttons)
				but.setEnabled(true);
			this.ident = iden;
			this.data_type = datatype;
			swt_table_protein swt = new swt_table_protein(table, data_type);
			ident.view_result(swt);
			// on mets à jour le texte du shell
			if (data_type.getIs_invididual())
				this.setText(ident.getMsRunSet().getMsRunList().get(0)
						.getSampleName());
			else if (!data_type.getIs_phosphopeptide())
				this.setText("Combine results");
			else
				this.setText("PhosphoIsland results");
			// on rempli la description
			if (this.data_type.getIs_phosphopeptide()) {
				description.setText("Phosphoisland groups : "
						+ ident.getGrouping().getGroupList().size()
						+ " - Phosphoisland subgroups : "
						+ swt.getSubgroup_count()
						+ " - Phosphoislands : "
						+ table.getItemCount());
			} else {
				description.setText("Protein groups : "
						+ ident.getGrouping().getGroupList().size()
						+ " - Protein subgroups : " + swt.getSubgroup_count()
						+ " - Proteins : "
						+ table.getItemCount());
			}
		} catch (Exception e) {
			window.view_error(e, "Problems with drawing protein list");
		}
	}

	public Identification getIdent() {
		return ident;
	}

	public void setIdent(Identification ident) {
		this.ident = ident;
	}

	public DataType_Base getData_type() {
		return data_type;
	}

	public void setData_type(DataType_Base data_type) {
		this.data_type = data_type;
	}

}
